;--------------------------------
;Include Modern UI

  !define MULTIUSER_EXECUTIONLEVEL Highest
  !define MULTIUSER_MUI
  !define MULTIUSER_INSTALLMODE_COMMANDLINE

  !include MultiUser.nsh
  !include "MUI2.nsh"
  !include "Sections.nsh"
;--------------------------------
; The name of the installer
Name "OptiLoading"

  !define JRE_VERSION "1.5.0"

; The file to write
OutFile "..\..\dist\win\setup_jre_incl.exe"

; The default installation directory
InstallDir C:\OptiLoading

; Registry key to check for directory (so if you install again, it will 
; overwrite the old one automatically)
InstallDirRegKey HKCU "Software\OptiLoading" "Install_Dir"

!define MUI_ICON "${NSISDIR}\Contrib\Graphics\Icons\win-install.ico"
!define MUI_UNICON "${NSISDIR}\Contrib\Graphics\Icons\win-uninstall.ico"

;--------------------------------
;Interface Configuration

  !define MUI_HEADERIMAGE
  !define MUI_HEADERIMAGE_BITMAP "header.bmp" ; optional
  !define MUI_ABORTWARNING

;--------------------------------
;Language Selection Dialog Settings

  ;Remember the installer language
  !define MUI_LANGDLL_REGISTRY_ROOT "HKCU" 
  !define MUI_LANGDLL_REGISTRY_KEY "Software\OptiLoading" 
  !define MUI_LANGDLL_REGISTRY_VALUENAME "Installer Language"

;--------------------------------

; Pages

  !insertmacro MUI_PAGE_WELCOME
  !insertmacro MUI_PAGE_LICENSE "..\..\LICENSE.txt"
  !insertmacro MULTIUSER_PAGE_INSTALLMODE

  Page custom CheckInstalledJRE
  !insertmacro MUI_PAGE_COMPONENTS
  !insertmacro MUI_PAGE_DIRECTORY
  !insertmacro MUI_PAGE_INSTFILES
  !define MUI_FINISHPAGE_SHOWREADME "$INSTDIR\README.txt"  
  !define MUI_FINISHPAGE_RUN "$INSTDIR\OptiLoading.exe"
  !insertmacro MUI_PAGE_FINISH  

  !insertmacro MUI_UNPAGE_WELCOME
  !insertmacro MUI_UNPAGE_CONFIRM
  !insertmacro MUI_UNPAGE_INSTFILES
  !insertmacro MUI_UNPAGE_FINISH


;--------------------------------
;Languages

  !insertmacro MUI_LANGUAGE "English" ;first language is the default language
  !insertmacro MUI_LANGUAGE "French"
  !insertmacro MUI_LANGUAGE "Dutch"

;--------------------------------

; The stuff to install
Section "OptiLoading (required)" SecAppFiles

  SectionIn RO
  
  ; Set output path to the installation directory.
  SetOutPath $INSTDIR
  
  ; Put file there
  File "..\..\dist\win\OptiLoading.exe"
  File "..\..\res\win\lib\glpk_jni.dll"
  File "..\..\res\win\lib\lpsolve55.dll"
  File "..\..\res\win\lib\lpsolve55j.dll"
  File "..\..\README.txt"
  
  ; Write the installation path into the registry
  WriteRegStr HKCU SOFTWARE\OptiLoading "Install_Dir" "$INSTDIR"
  
  ; Write the uninstall keys for Windows
  WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\OptiLoading" "DisplayName" "OptiLoading"
  WriteRegStr HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\OptiLoading" "UninstallString" '"$INSTDIR\uninstall.exe"'
  WriteRegDWORD HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\OptiLoading" "NoModify" 1
  WriteRegDWORD HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\OptiLoading" "NoRepair" 1
  WriteUninstaller "uninstall.exe"
  
SectionEnd

; Optional section (can be disabled by the user)
Section "Start Menu Shortcuts"  SecCreateShortcut

  CreateDirectory "$SMPROGRAMS\OptiLoading"
  CreateShortCut "$SMPROGRAMS\OptiLoading\Uninstall.lnk" "$INSTDIR\uninstall.exe" "" "$INSTDIR\uninstall.exe" 0
  CreateShortCut "$SMPROGRAMS\OptiLoading\ReadMe.lnk" "$INSTDIR\README.txt" "" "$INSTDIR\README.txt" 0
  CreateShortCut "$SMPROGRAMS\OptiLoading\OptiLoading.lnk" "$INSTDIR\OptiLoading.exe" "" "$INSTDIR\OptiLoading.exe" 0
  
SectionEnd

Section "JRE" jre

    File /r "..\..\res\win\jre"
  
SectionEnd

;--------------------------------
;Installer Functions

Function .onInit

  !insertmacro MULTIUSER_INIT
  !insertmacro MUI_LANGDLL_DISPLAY

FunctionEnd

Function CheckInstalledJRE
  Push "${JRE_VERSION}"
  Call DetectJRE
  Exch $0	; Get return value from stack
  StrCmp $0 "0" NoFound
  StrCmp $0 "-1" FoundOld
  Goto JREAlreadyInstalled
 
FoundOld:
  Call SelectJRE
  Return
  
NoFound:
  Call SelectJRE
  Return
 
JREAlreadyInstalled:
  Call UnselectJRE
  Return
 
FunctionEnd
 
; Returns: 0 - JRE not found. -1 - JRE found but too old. Otherwise - Path to JAVA EXE
 
; DetectJRE. Version requested is on the stack.
; Returns (on stack)	"0" on failure (java too old or not installed), otherwise path to java interpreter
; Stack value will be overwritten!
 
Function DetectJRE
  Exch $0	; Get version requested  
		; Now the previous value of $0 is on the stack, and the asked for version of JDK is in $0
  Push $1	; $1 = Java version string (ie 1.5.0)
  Push $2	; $2 = Javahome
  Push $3	; $3 and $4 are used for checking the major/minor version of java
  Push $4
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment" "CurrentVersion"
  StrCmp $1 "" DetectTry2
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Runtime Environment\$1" "JavaHome"
  StrCmp $2 "" DetectTry2
  Goto GetJRE
 
DetectTry2:
  ReadRegStr $1 HKLM "SOFTWARE\JavaSoft\Java Development Kit" "CurrentVersion"
  StrCmp $1 "" NoFound
  ReadRegStr $2 HKLM "SOFTWARE\JavaSoft\Java Development Kit\$1" "JavaHome"
  StrCmp $2 "" NoFound
 
GetJRE:
; $0 = version requested. $1 = version found. $2 = javaHome
  IfFileExists "$2\bin\java.exe" 0 NoFound
  StrCpy $3 $0 1			; Get major version. Example: $1 = 1.5.0, now $3 = 1
  StrCpy $4 $1 1			; $3 = major version requested, $4 = major version found
  IntCmp $4 $3 0 FoundOld FoundNew
  StrCpy $3 $0 1 2
  StrCpy $4 $1 1 2			; Same as above. $3 is minor version requested, $4 is minor version installed
  IntCmp $4 $3 FoundNew FoundOld FoundNew
 
NoFound:
  Push "0"
  Goto DetectJREEnd
 
FoundOld:
;  Push ${TEMP2}
  Push "-1"
  Goto DetectJREEnd  
  
FoundNew:
  Push "$2\bin\java.exe"
;  Push "OK"
;  Return
   Goto DetectJREEnd
   
DetectJREEnd:
	; Top of stack is return value, then r4,r3,r2,r1
	Exch	; => r4,rv,r3,r2,r1,r0
	Pop $4	; => rv,r3,r2,r1r,r0
	Exch	; => r3,rv,r2,r1,r0
	Pop $3	; => rv,r2,r1,r0
	Exch 	; => r2,rv,r1,r0
	Pop $2	; => rv,r1,r0
	Exch	; => r1,rv,r0
	Pop $1	; => rv,r0
	Exch	; => r0,rv
	Pop $0	; => rv 
FunctionEnd

Function SelectJRE
  !insertmacro SelectSection ${jre}
FunctionEnd
 
Function UnselectJRE
  !insertmacro UnselectSection ${jre}
FunctionEnd


;--------------------------------

; Uninstaller

Section "Uninstall"
  
  ; Remove registry keys
  DeleteRegKey HKCU "Software\Microsoft\Windows\CurrentVersion\Uninstall\OptiLoading"
  DeleteRegKey HKCU SOFTWARE\OptiLoading

  ; Remove shortcuts, if any
  Delete "$SMPROGRAMS\OptiLoading\*.*"

  ; Remove directories used
  RMDir /r /REBOOTOK "$INSTDIR"
  RMDir "$SMPROGRAMS\OptiLoading"

SectionEnd


;--------------------------------
;Uninstaller Functions

Function un.onInit

  !insertmacro MULTIUSER_UNINIT
  !insertmacro MUI_UNGETLANGUAGE
  
FunctionEnd