//: be/optiloading/cargo/AddFrame.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ResourceBundle;

import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JDialog;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;

import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * AddFrame is the dialogbox to add cargo.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class AddFrame extends JDialog{
	private UpperCaseField inputid;
	private JFormattedTextField inputcargo;
	private JSpinner density;
	private JTextField inputdescription;
	private JFormattedTextField densityfield;
	private JComboBox volumeorweight;
	private static AddFrame uniqueInstance;
	private ResourceBundle languageResource;
	
	/**
	 * Get singleton instance.
	 * 
	 * @return Log object.
	 */
	public static AddFrame getInstance(){
		if (uniqueInstance == null) {
			uniqueInstance = new AddFrame();
		}
		return uniqueInstance;
	}
	
	/**
	 * Check if addFrame exists.
	 * @return true if it does not exist.
	 */
	public static boolean isNull(){
		if (uniqueInstance == null) {
			return true;
		} else {
			return false;
		}
	}
	
	//Private constructor
	private AddFrame(){
		super(MainFrame.getInstance());
		languageResource = Settings.getInstance().getResourceBundle();
		this.setTitle(languageResource.getString("addCargo"));
		
		// Input Panel
		JPanel input = new JPanel(new GridBagLayout());
		input.setBorder(new CompoundBorder(new EtchedBorder()
										,new EmptyBorder(10, 10, 10, 10)));
		KeyAdapter keylistener = new KeyAdapter(){
			public void keyPressed(KeyEvent e) {
				if(e.getKeyCode() == KeyEvent.VK_ENTER){
					try {
						inputcargo.commitEdit();
						densityfield.commitEdit();
					} catch (ParseException ex) {
						Log.getInstance().add(Log.JAVA,ex);
					}
					addCargo(true);
				}
			}
		};
		
		//GridBagLayout Settings
		Dimension shortField = new Dimension( 120, 22 );
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;

		//ID
		JLabel id = new JLabel(" " + languageResource.
				getString("id")+":");
		input.add(id,c);
		inputid = new UpperCaseField();
		inputid.setHorizontalAlignment(JTextField.RIGHT);
		inputid.setToolTipText(languageResource.
				getString("descriptionTooltip"));
		inputid.addKeyListener(keylistener);
		inputid.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(inputid, c);
		
		//Description
		JLabel description = new JLabel(" " + languageResource.
				getString("description")+":");
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
		input.add(description,c);
		inputdescription = new JTextField();
		inputdescription.setHorizontalAlignment(JTextField.RIGHT);
		inputdescription.setToolTipText(languageResource.
				getString("descriptionTooltip"));
		inputdescription.addKeyListener(keylistener);
		inputdescription.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(inputdescription, c);
		
		//Volume or weight
		String[] volumeorweightlist = { languageResource.getString("volumeLabel"), 
		languageResource.getString("weight")+":", languageResource.getString("tmd")+":"};
		volumeorweight = new JComboBox(volumeorweightlist);
		volumeorweight.setToolTipText(languageResource.getString("inputTooltip"));
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
		input.add(volumeorweight,c);
		inputcargo = new JFormattedTextField(NumberFormat.getInstance());
		inputcargo.setHorizontalAlignment(JTextField.RIGHT);
		inputcargo
				.setToolTipText(languageResource.getString("inputFieldTooltip"));
		inputcargo.addKeyListener(keylistener);
		inputcargo.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(inputcargo, c);
		
		//Density
		JLabel densitylabel = new JLabel(languageResource.getString("densityLabel"));
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
		input.add(densitylabel,c);
		SpinnerNumberModel densitymod = new SpinnerNumberModel(1.000,
				0.001, 22.590, 0.001);
		density = new JSpinner(densitymod);
		density.setToolTipText(languageResource.getString("densityFieldTooltip"));
		density.setEditor(new JSpinner.NumberEditor(density, "0.000"));
		densityfield = ((JSpinner.DefaultEditor) density.getEditor())
				.getTextField();
		densityfield.setFont(new Font("Dialog", Font.PLAIN, 12));
		densityfield.addKeyListener(keylistener);
		density.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(density, c);

		JButton addButton = new JButton(languageResource.getString("addLabel"));
		addButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				addCargo(true);
			}
		});
		
		JButton addNextButton = new JButton(languageResource.getString("addAnotherLabel"));
		addNextButton.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				addCargo(false);
			}
		});
		
		JButton cancelButton = new JButton(languageResource.getString("cancelLabel"));
		cancelButton.addActionListener(new ActionListener(){

			public void actionPerformed(ActionEvent arg0) {
				terminate();
			}
			
		});
		
		JPanel buttonPanel = new JPanel();
		buttonPanel.add(addButton);
		buttonPanel.add(addNextButton);
		buttonPanel.add(cancelButton);
		
		JPanel main = new JPanel(new BorderLayout());
		main.setBorder(new EmptyBorder(5,5,5,5));
		main.add(BorderLayout.CENTER,input);
		main.add(BorderLayout.SOUTH,buttonPanel);
		getContentPane().add(main);
		setResizable(false);
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	//Method to add cargo to the CargoList
	public void addCargo(boolean terminate) {
		ShipData data = null;
		
		if (inputid.getText().equals("")){
			JOptionPane.showMessageDialog(
					this,languageResource.getString("messageId"));
			return;
		} else if(inputdescription.getText().equals("")){
			JOptionPane.showMessageDialog(
					this,languageResource.getString("messageDescription"));
			return;
		} else if (ShipData.getInstance().getCargoList()
				.checkId(inputid.getText())){
			JOptionPane.showMessageDialog(
					this,languageResource.getString("messageIdinuse"));
			return;
		} else if (inputcargo.getValue()==null){
			JOptionPane.showMessageDialog(
					this,languageResource.getString("messageInput"));
			return;
		}else if( ((Number)inputcargo.getValue()).floatValue() <= 0f){
			String text = null;
			if(volumeorweight.getSelectedIndex() == 0){
				text = languageResource.getString("volume2");
			}else if(volumeorweight.getSelectedIndex() == 1){
				text=languageResource.getString("weight2");
			}else{
				text="TMD";
			}
			JOptionPane.showMessageDialog(
					this,languageResource.getString("notvalid")+" "+text+".");
			return;
		}else if (volumeorweight.getSelectedIndex() == 2){
			float input = ((Number) inputcargo.getValue()).floatValue();
			data = ShipData.getInstance();
			if((input > data.getMaxdraught()) || (input < data.getMindraught())){
				NumberFormat formatter = NumberFormat.getInstance();
		        formatter.setMaximumFractionDigits(2);
		        formatter.setMinimumFractionDigits(2);
				JOptionPane.showMessageDialog(this, String.format(
						languageResource.getString("messageTMDBetween"),
						formatter.format(data.getMindraught()),
						formatter.format(data.getMaxdraught())));
				return;
			}
		}
		
		Number density = (Number) densityfield.getValue();
		Number volorweight = (Number) inputcargo.getValue();
		float weight = 0;
		if (volumeorweight.getSelectedIndex() == 0){
			weight = volorweight.floatValue() * density.floatValue();
		} else if(volumeorweight.getSelectedIndex() == 1){
			weight = volorweight.floatValue();
		} else {
			weight = data.getDeadweight(volorweight.floatValue()-0.0001f);
			if (weight <= 0f){
				NumberFormat formatter = NumberFormat.getInstance();
		        formatter.setMaximumFractionDigits(2);
		        formatter.setMinimumFractionDigits(2);
				JOptionPane.showMessageDialog(this, String.format(
						languageResource.getString("messageTMDMin"),
						formatter.format(weight)));
				return;
			}
		}
		
		Cargo cargoObject = new Cargo(inputid.getText(),inputdescription.getText(),weight,
				density.floatValue());
		
		if(ShipData.getInstance().getCargoList().add(cargoObject,this) == true){
			ShipData.getInstance().getCargoCompList().addComp();
			ShipData.getInstance().getTankCompList().addComp();
			MainFrame.getInstance().updateInterface();
			
			if(terminate == true){
				terminate();
			} else {
				inputcargo.setValue(null);
				inputdescription.setText("");
				inputid.setText("");
				densityfield.setValue(null);
				inputid.requestFocus();
			}
		}
	}

	//Dispose of the addCargo dialog
	public void terminate(){
		this.setVisible(false);
		this.dispose();
		uniqueInstance = null;
	}
}

