//: /be/optiloading/cargo/Cargo.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

/**
 * Cargo class used to stow information about the cargo
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Cargo {
	private String description;
	private String cargoid;
	private float weight;
	private float volume;
	private float density;

	/**
	 * Constructor used to create this object. Responsible for setting the
	 * correct weight, volume and specific gravity.
	 * 
	 * @param description
	 *            The cargo description.
	 * @param weight
	 *            A floating point value indicating the weight of the cargo in
	 *            tonnes.
	 * @param density
	 *            The specific weight of the cargo.
	 */
	public Cargo(String cargoid, String description, float weight, float density) {
		this.volume = weight / density;
		this.weight = weight;
		this.density = density;
		this.description = description;
		this.cargoid = cargoid;
	}

	/**
	 * Returns the weight of the cargo in tonnes. The weight is set within the
	 * constructor.
	 * 
	 * @return Floating point value that indicates the weight of the cargo in
	 *         tonnes.
	 */
	public float getWeight() {
		return weight;
	}

	/**
	 * Returns the volume of the cargo in m^3 . The volume is calculated within
	 * the constructor.
	 * 
	 * @return Floating point value that indicates the volume of the cargo in
	 *         m^3.
	 */
	public float getVolume() {
		return volume;
	}
	
	/**
	 * Returns the desscription of the cargo.
	 * 
	 * @return String value indicating the description of the cargo.
	 */
	public String getDescription() {
		return description;
	}

	/**
	 * Returns the apparent density of the cargo. 
	 * The apparent density is set within the constructor.
	 * 
	 * @return Floating point indicating the apparent density.
	 */
	public float getDensity() {
		return density;
	}
	
	/**
	 * Returns the cargo id 
	 * 
	 * @return String containing the cargo id.
	 */
	public String getCargoid() {
		return cargoid;
	}
	
	/**
	 * Overwritten method which returns the discription of the cargo;
	 * 
	 * @return String indicating the discription of the cargo.
	 */
	public String toString(){
		return description;
	}

	/**
	 * Method to set the description.
	 * 
	 * @param description String value containing the description.
	 */
	public void setDescription(String description) {
		this.description = description;
	}
	
	/**
	 * Method to set the cargo id.
	 * 
	 * @param id String value containing the cargo id.
	 */
	public void setCargoid(String id) {
		this.cargoid = id;
	}

	/**
	 * Method to set the weight.
	 * 
	 * @param weight Float value containing the weight.
	 */
	public void setWeight(float weight) {
		this.weight = weight;
		volume = weight / density;
	}

	/**
	 * Method to set the volume.
	 * 
	 * @param volume Float value containing the volume.
	 */
	public void setVolume(float volume) {
		this.volume = volume;
		weight = volume * density;
	}

	/**
	 * Method to set the density.
	 * 
	 * @param density Float value containing the density.
	 */
	public void setDensity(float density) {
		this.density = density;
		weight = density * volume;
	}
}