//: be/optiloading/cargo/InputCargo.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;
import javax.swing.table.AbstractTableModel;

import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * CargoModel is a TableModel to build tables out of the ArrayList<Cargo>.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */


@SuppressWarnings("serial")
public class CargoModel extends AbstractTableModel{
	private String[] columnNames;
	private CargoList cargoList;
	private boolean editable;
	private NumberFormat formatter;
	private ResourceBundle languageResource;
	
	/**
	 * Constructor
	 * @param cargoList The cargoList which contains the data.
	 */
	public CargoModel(CargoList cargoList){
		this.cargoList = cargoList;
		editable = true;
		languageResource = Settings.getInstance().getResourceBundle();
		columnNames = new String[6];
		columnNames[0] = "#";
		columnNames[1] = languageResource.getString("id");
		columnNames[2] = languageResource.getString("description");
		columnNames[3] = languageResource.getString("volume");
		columnNames[4] = languageResource.getString("weight");
		columnNames[5] = languageResource.getString("density");
		formatter = NumberFormat.getInstance();
        formatter.setMaximumFractionDigits(3);
        formatter.setMinimumFractionDigits(3);
	}
	
	/**
	 * Get the number of columns.
	 * 
	 * @return Integer indicating the number of columns.
	 */
	public int getColumnCount() {
		return columnNames.length;
	}

	/**
	 * Get the number of rows.
	 * 
	 * @return Integer indicating the number of rows.
	 */
	public int getRowCount() {
		return cargoList.size();
	}
	
	/**
	 * Get the column name.
	 * 
	 * @param col Specification of which column.
	 */
	public String getColumnName(int col) {
		return columnNames[col];
	}
	
	/**
	 * Get the class of of a certain columns.
	 * 
	 * @param column Column for which you want the class.
	 * @return Class of the certain column.
	 */
	public Class<?> getColumnClass(int column){
		if(column == 0){
			return Integer.class;
		} else if (column > 2) {
			return Float.class;
		} else {
			return String.class;
		}
	}

	/**
	 * Get the value of a specific cell.
	 * 
	 * @param arg0 Row of the cell.
	 * @param arg1 Column of the cell.
	 * @return Object value of the cell.
	 */
	public Object getValueAt(int arg0, int arg1) {
		if(arg0 >= cargoList.size()){
			return null;
		}
		Cargo cargoObject = cargoList.get(arg0);
		Object returnObject = null;
		switch(arg1){
		case 0:
			returnObject = new Integer(arg0+1);
			break;
		case 1:
			returnObject = cargoObject.getCargoid();
			break;
		case 2:
			returnObject = cargoObject.getDescription();
			break;
		case 3:
			returnObject = cargoObject.getVolume();
			break;
		case 4:
			returnObject = cargoObject.getWeight();
			break;
		case 5:
			returnObject = cargoObject.getDensity();
			break;
		}
		return returnObject;
	}
	
	public boolean isCellEditable(int row,int column){
		if(column == 0){
			return false;
		}
		return editable;
	}
	
	/**
	 * Set the value of a specific cell.
	 * 
	 * @param row Row of the cell.
	 * @param column Column of the cell.
	 */
	public void setValueAt(Object objectvalue, int row, int column){
		Cargo current = cargoList.get(row);
		ShipData data = ShipData.getInstance();
		
		float maxvolume = data.getMaxcargovolume() - cargoList.getTotalVolume() 
		+ current.getVolume();
		float maxweight = (data.getDeadweightleft() + current.getWeight());
		
		float fvalue = 0;
		String svalue = null;
		
		if(column > 2){
			Number number = (Number) objectvalue;
			fvalue = number.floatValue();
		} else{
			svalue = (String) objectvalue;
		}

		switch(column){
		case 1:
			if (!checkID(svalue, row)) {
				data.getCargoTankList().changeID(current.getCargoid(), svalue);
				current.setCargoid(svalue);
			} else {
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						languageResource.getString("messageIdinuse"));
			}
			break;
		case 2:
			current.setDescription(svalue);
			break;
		case 3:
			if(fvalue>maxvolume){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(languageResource.getString("messageMaxVolume"),
						formatter.format(maxvolume),
						formatter.format(data.getMaxcargovolume())));
			} else if((current.getDensity()*fvalue)>maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance()
						,String.format(
						languageResource.getString("messageMaxWeight"),
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else if(cargoList.getMinTanks(row, fvalue)>
				ShipData.getInstance().getNumberofcargotanks()){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						languageResource.getString("messageMaxTanks"));
			} else {
				current.setVolume(fvalue);
			}
			break;
		case 4:
			if(fvalue > maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(languageResource.getString("messageMaxWeight"),
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else if( (fvalue/current.getDensity()) > maxvolume) {
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(languageResource.getString("messageMaxVolume"),
						formatter.format(maxvolume),
						formatter.format(data.getMaxcargovolume())));
			} else {
				current.setWeight(fvalue);
			}
			break;
		case 5:
			if((fvalue*current.getVolume())>maxweight){
				JOptionPane.showMessageDialog(MainFrame.getInstance(),
						String.format(languageResource.getString("messageMaxWeight"),
						formatter.format(maxweight),
						formatter.format(data.getMaxdeadweight())));
			} else {
				current.setDensity(fvalue);
			}
			break;
		}
		
		
		fireTableDataChanged();
		data = null;
		MainFrame.getInstance().updateInterface();
	}
	
	/**
	 * Make tableModel editable.
	 * @param editable True if editable.
	 */
	public void setEditable(boolean editable){
		this.editable = editable;
	}
	
	// Check if id is already in use
	private boolean checkID(String id, int cargo){
		boolean returnvalue = false;
		for(int i = 0; i < cargoList.size(); i++){
			if((i != cargo) && (cargoList.get(i).getCargoid().equals(id))){
				returnvalue = true;
			}
		}
		return returnvalue;
	}
	
}
