//: be/optiloading/cargo/UppercaseField.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.cargo;

import javax.swing.JTextField;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.PlainDocument;

/**
 * UpperCaseField is used for the cargo ids.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class UpperCaseField extends JTextField {
	 
	/**
	 * Constructor
	 */
    public UpperCaseField() {
        super();
    }
 
    /**
     * Document
     */
    protected Document createDefaultModel() {
 	     return new UpperCaseDocument();
    }
 
    /**
     * UpperCase Document
     */
    @SuppressWarnings("serial")
	private class UpperCaseDocument extends PlainDocument {
 
    	public void insertString(int offs, String str, AttributeSet a) 
 	    	throws BadLocationException {
        	 
    		if (str == null) {
    		return;
 	        }
 	        char[] upper = str.toCharArray();
 	        for (int i = 0; i < upper.length; i++) {
 	        	upper[i] = Character.toUpperCase(upper[i]);
 	        }
 	       String old = getText( 0, getLength() );
 	      
 	       String newStr = old.substring(0, offs) + str + old.substring(offs);
 	 
 	 
 	       boolean valid = isValid(newStr);
 	 
 	       if (valid) {
 	    	   super.insertString( offs, new String(upper), a );
 	       }
    	}
    	
    	// Cargo id can only have 3 characters
    	public boolean isValid(String str) {
    	   if(str.length() > 3){
    		   return false;
    	   } else {
    		   return true;
    	   }
    	}
    }
}
