//: /be/optiloading/compatibility/CargoComp.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.compatibility;

import java.util.ArrayList;

/**
 * CargoComp class used to stow information about the cargo-cargo compatibility.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class CargoComp extends ArrayList<ArrayList<Boolean>> {
	
	/**
	 * Get a specific compatibility value.
	 * @param row Integer with row value.
	 * @param col Integer with column value.
	 * @return Boolean which is false when compatible, otherwise true.
	 */
	public boolean get(int row, int col){
		ArrayList<Boolean> getList = this.get(row);
		return getList.get(col);
	}
	
	/**
	 * Add a compatibility when load is added.
	 */
	public void addComp(){
		int currentSize = this.size();
		ArrayList<Boolean> newItem = new ArrayList<Boolean>();
		for(int i = 0; i < currentSize; i++){
			newItem.add(new Boolean(false));
			this.get(i).add(new Boolean(false));
		}
		newItem.add(new Boolean(false));
		this.add(newItem);
	}
	
	/**
	 * Remove compatibility when load is removed.
	 * @param index Integer specifies which load to remove.
	 */
	public void deleteComp(int index){
		for(int i = 0; i < this.size();i++){
			this.get(i).remove(index);
		}
		this.remove(index);
	}
	
	/**
	 * Set a specific compatibility value
	 * @param value Boolean which is false when compatible, otherwise true.
	 * @param horz Integer with horizontal value.
	 * @param vert Integer with vertical value.
	 */
	public void set(boolean value,int horz, int vert){
		this.get(horz).set(vert, value);
	}
}
