//: /be/optiloading/compatibility/CargoCompModel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.compatibility;

import javax.swing.table.AbstractTableModel;

import be.optiloading.cargo.CargoList;
import be.optiloading.gui.MainFrame;

/**
 * CargoCompModel is a TableModel to build tables out of the CargoComp object.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class CargoCompModel extends AbstractTableModel{
	private CargoComp cargoCompList;
	private CargoList cargoList;
	
	public CargoCompModel(CargoComp cargoCompList, CargoList cargoList){
		this.cargoCompList = cargoCompList;
		this.cargoList = cargoList;
	}
	
	/**
	 * Get the number of columns.
	 * 
	 * @return Integer indicating the number of columns.
	 */
	public int getColumnCount() {
		return (cargoList.size() +1);
	}

	/**
	 * Get the number of rows.
	 * 
	 * @return Integer indicating the number of rows.
	 */
	public int getRowCount() {
		return cargoList.size();
	}
	
	/**
	 * Get the column name.
	 * 
	 * @param col Specification of which column.
	 */
	public String getColumnName(int col) {
		String returnvalue = null;
		if(col != 0){
			returnvalue = cargoList.get(col-1).getCargoid();
		}
		return returnvalue;
	}
	
	/**
	 * Get the class of of a certain columns.
	 * 
	 * @param column Column for which you want the class.
	 * @return Class of the certain column.
	 */
	public Class<?> getColumnClass(int column){
		if(column > 0){
			return Boolean.class;
		} else {
			return String.class;
		}
	}

	/**
	 * Get the value of a specific cell.
	 * 
	 * @param row Row of the cell.
	 * @param col Column of the cell.
	 * @return Object value of the cell.
	 */
	public Object getValueAt(int row, int col) {
		if(row >= cargoList.size()){
			return null;
		}
		
		Object returnObject = null;
		switch(col){
		case 0:
			returnObject = cargoList.get(row).getCargoid();
			break;
		default:
			returnObject = cargoCompList.get(row, col-1);
		}
		return returnObject;
	}
	
	/**
	 * Returns if cell is editable
	 * @param row Integer with row number.
	 * @param col Integer with col number.
	 */
	public boolean isCellEditable(int row,int col){
		boolean returnvalue = true;
		if((col == 0) || ((col-1) == row)){
			returnvalue = false;
		}
		return returnvalue;
	}
	
	/**
	 * Set the value of a specific cell.
	 * 
	 * @param row Row of the cell.
	 * @param column Column of the cell.
	 */
	public void setValueAt(Object objectvalue, int row, int column){
		
		boolean bvalue = (Boolean) objectvalue;
		
		cargoCompList.set(bvalue, row, column-1);
		cargoCompList.set(bvalue, column-1, row);
		
		
		fireTableDataChanged();
		MainFrame.getInstance().updateInterface();
	}
	
}
