//: /be/optiloading/generalinfo/InfoModel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.generalinfo;

import java.util.ResourceBundle;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;

import be.optiloading.helpers.tree.TreeTableModel;
import be.optiloading.settings.Settings;

/**
 * Model class for the TreeTable. Child of teh DefaultTreeModel.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
class InfoModel extends DefaultTreeModel 
                             implements TreeTableModel {
	
    protected DefaultMutableTreeNode root;
	
    static protected String[]  cNames;

    static protected Class<?>[]  cTypes = {TreeTableModel.class, Float.class};

    private ResourceBundle languageResource;
    
    /**
     * Constructor.
     * 
     * @param root The top node of the tree.
     */
    public InfoModel(DefaultMutableTreeNode root) { 
    	super(root); 
    	this.root = root;
    	
    	languageResource = Settings.getInstance().getResourceBundle();
    	
    	cNames = new String[2];
    	cNames[0] = languageResource.getString("description");
    	cNames[1] = languageResource.getString("value");
    	
    	createCategories();
    }
    
    //Private method to create the categories.
	private void createCategories(){
		DefaultMutableTreeNode category = null;
		
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("conditionCategory")));
		root.add(category);
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("cargoCategory")));
		root.add(category);
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("ballastCategory")));
		root.add(category);
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("draughtCategory")));
		root.add(category);
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("stabilityCategory")));
		root.add(category);
		category = new DefaultMutableTreeNode(new InfoObject(languageResource.getString("strengthCategory")));
		root.add(category);
	}

	/**
	 * Get the number of children.
	 * 
	 * @param node Object for which you want the childcount.
	 * @return int value containing the childcount.
	 */
    public int getChildCount(Object node) { 
    	DefaultMutableTreeNode treenode = (DefaultMutableTreeNode) node;
    	return treenode.getChildCount();
    }

    /**
     * Get the child of an object.
     * 
     * @param node Node for which you want the child.
     * @param i index.
     * @retun child node.
     */
    public Object getChild(Object node, int i) { 
    	DefaultMutableTreeNode treenode = (DefaultMutableTreeNode) node;
    	return treenode.getChildAt(i);
    }

    /**
     * Is this node a leaf.
     * 
     * @param node object.
     * @return boolean value specifying if this is a leaf.
     */
    public boolean isLeaf(Object node) { 
    	DefaultMutableTreeNode treenode = (DefaultMutableTreeNode) node;
    	return treenode.isLeaf();
    }

    /**
     * Number of columns.
     * 
     * @return int with columncount.
     */
    public int getColumnCount() {
    	return cNames.length;
    }

    /**
     * Get column name.
     * 
     * @param column int index value.
     * @return String with the name.
     */
    public String getColumnName(int column) {
    	return cNames[column];
    }

    /**
     * Get column class.
     * 
     * @param column int index value.
     * @return Class with the class.
     */
    public Class<?> getColumnClass(int column) {
    	return cTypes[column];
    }
 
    /**
     * Get the value at of an node.
     * 
     * @param node Object.
     * @param column Integer column.
     * @return Object node.
     */
    public Object getValueAt(Object node, int column) {
		DefaultMutableTreeNode info = (DefaultMutableTreeNode) node;
		Object nodeInfo = info.getUserObject();
		InfoObject infoobject = (InfoObject) nodeInfo;
    	return infoobject.get(column);
    }

    /**
     * Get root node.
     * 
     * @return Treenode root.
     */
    public TreeNode getRoot() {
        return root;
    }

    /**
     * Is cell editable.
     * 
     * @return boolean.
     */
    public boolean isCellEditable(Object node, int column) { 
        return getColumnClass(column) == TreeTableModel.class;  
    }

    /**
     * Set value at.
     * 
     * @param aValue Object value.
     * @param node Object node.
     * @param column int column.
     */
   public void setValueAt(Object aValue, Object node, int column) {}

}


