//: /be/optiloading/generalinfo/InfoObject.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.generalinfo;

import be.optiloading.ship.ShipData;
import be.optiloading.ship.ShipParticular;

/**
 * Info objects that will be displayed in the treetable.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class InfoObject {
	private String description;
	private ShipParticular shipparticular;
	private int category;
	private boolean cat;
	
	/**
	 * Constructor.
	 * 
	 * @param description Description of the info.
	 * @param shipparticular ShipParticular that needs to be displayed.
	 * @param category To which category this info belongs.
	 */
	public InfoObject(String description, ShipParticular shipparticular,
			int category){
		this.description = description;
		this.shipparticular = shipparticular;
		this.category = category;
		cat = false;
	}
	
	/**
	 * Constructor without specific value (for category values).
	 * 
	 * @param description Description of the info.
	 */
	public InfoObject(String description){
		this.description = description;
		cat = true;
	}
	
	/**
	 * Get the value of a specific column.
	 * 
	 * @param i int which column value.
	 * @return Object value.
	 */
	public Object get(int i){
		Object returnvalue = null;
		switch(i){
		case 0: 
			returnvalue = description;
			break;
		case 1:
			if(cat == true){
				returnvalue = null;
			} else {
				returnvalue = ShipData.getInstance().
				getShipParticular(shipparticular);
			}
			break;
		}
		return returnvalue;
	}
	
	/**
	 * Overwritten toString.
	 * 
	 * @return Description.
	 */
	public String toString(){
		return description;
	}

	/**
	 * getCatgory()
	 * 
	 * @return InfoPanel constant indicating the category.
	 */
	public int getCategory(){
		return category;
	}
}