//: /be/optiloading/generalinfo/nfoPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.generalinfo;

import java.awt.BorderLayout;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.util.ArrayList;
import java.util.ResourceBundle;

import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.table.TableColumn;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreePath;

import be.optiloading.helpers.table.NumberRenderer;
import be.optiloading.helpers.tree.JTreeTable;
import be.optiloading.helpers.tree.TreeTableModel;
import be.optiloading.helpers.tree.TreeTableModelAdapter;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipParticular;

/**
 * Panel containing the scrollPane and TreeTable.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class InfoPanel extends JPanel{
	public static final int CONDITION = 0;
	public static final int CARGO = 1;
	public static final int BALLAST = 2;
	public static final int DRAUGHT = 3;
	public static final int STABILITY = 4;
	public static final int STRENGTH = 5;
	
	private JTreeTable treetable;
	private JScrollPane scroll;
	private JTree tree;
	private InfoModel model;
	private DefaultMutableTreeNode top;
	
	private static InfoPanel uniqueInstance;
	private ResourceBundle languageResource;
	
	private ArrayList<InfoObject> infoList;
	
	/**
	 * Get an instance of the singleton.
	 * 
	 * @return InfoPanel object.
	 */
	public static InfoPanel getInstance(){
		if (uniqueInstance == null) {
			uniqueInstance = new InfoPanel();
		}
		return uniqueInstance;
	}
	
	//Singleton class constructor.
	private InfoPanel(){
		super(new BorderLayout());
		languageResource = Settings.getInstance().getResourceBundle();
		
		InfoObject root = new InfoObject(languageResource.getString("generalInfo"));
		top = new DefaultMutableTreeNode(root);
		model = new InfoModel(top);
		treetable = new JTreeTable(model);
	
		tree = (JTree) treetable.getDefaultRenderer(TreeTableModel.class);

		treetable.setDefaultRenderer(Float.class, new NumberRenderer(3));
		treetable.addFocusListener(new FocusAdapter(){
			public void focusLost(FocusEvent arg0) {
				((TreeTableModelAdapter)treetable.getModel())
						.fireTableDataChanged();
				
			}
		});
		
		TableColumn col = treetable.getColumnModel().getColumn(0);
		col.setMinWidth(185);
		col = treetable.getColumnModel().getColumn(1);
		col.setMinWidth(65);
		
		scroll = new JScrollPane(treetable, 
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.addComponentListener(new ScrollResizeListener());
		add(BorderLayout.CENTER,scroll);
		
		buildTree();
		buildTreeTable();
	}
	
	/**
	 * Add an InfoObject to the treetble.
	 * 
	 * @param category Flag indicating which category.
	 * @param info the new InfoObject.
	 */
	public void addInfo(int category, InfoObject info){
    	DefaultMutableTreeNode categoryNode = 
			(DefaultMutableTreeNode) model.getRoot().getChildAt(category);
    	DefaultMutableTreeNode infoNode = new DefaultMutableTreeNode(info);

		model.insertNodeInto(infoNode, categoryNode, categoryNode.getChildCount());
		update();
		tree.scrollPathToVisible(new TreePath(infoNode.getPath()));
    }
	
	private void buildTree(){
		infoList = new ArrayList<InfoObject>();
		infoList.add(new InfoObject(languageResource.getString("waterdensNode"),
				ShipParticular.DENSITY,CONDITION));
		infoList.add(new InfoObject(languageResource.getString("displacementNode"),
				ShipParticular.DISPLACEMENT,CONDITION));
		infoList.add(new InfoObject(languageResource.getString("deadweightNode"),
				ShipParticular.DEADWEIGHT,CONDITION));
		infoList.add(new InfoObject(languageResource.getString("deadweightleftNode"),
				ShipParticular.DEADWEIGHTLEFT,CONDITION));
		infoList.add(new InfoObject(languageResource.getString("totalvolumeNode"),
				ShipParticular.CARGO_VOLUME,CARGO));
		infoList.add(new InfoObject(languageResource.getString("totalvolumeleftNode"),
				ShipParticular.CARGO_VOLUME_LEFT,CARGO));
		infoList.add(new InfoObject(languageResource.getString("totalweightNode"),
				ShipParticular.CARGO_WEIGHT,CARGO));
		infoList.add(new InfoObject(languageResource.getString("totalvolumeNode"),
				ShipParticular.BALLAST_VOLUME,BALLAST));
		infoList.add(new InfoObject(languageResource.getString("totalweightNode"),
				ShipParticular.BALLAST_WEIGHT,BALLAST));
		infoList.add(new InfoObject(languageResource.getString("trimNode"),
				ShipParticular.TRIM,DRAUGHT));
		infoList.add(new InfoObject(languageResource.getString("tmdNode"),
				ShipParticular.TMD,DRAUGHT));
		infoList.add(new InfoObject(languageResource.getString("dfwdNode"),
				ShipParticular.DFWD,DRAUGHT));
		infoList.add(new InfoObject(languageResource.getString("daftNode"),
				ShipParticular.DAFT,DRAUGHT));
		infoList.add(new InfoObject(languageResource.getString("heelNode"),
				ShipParticular.HEEL,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("gmNode"),
				ShipParticular.GM,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("fscNode"),
				ShipParticular.FSC,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("vcgNode"),
				ShipParticular.VCG,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("lcgNode"),
				ShipParticular.LCG,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("tcgNode"),
				ShipParticular.TCG,STABILITY));
		infoList.add(new InfoObject(languageResource.getString("maxsfNode"),
				ShipParticular.ABS_MAX_SF,STRENGTH));
		infoList.add(new InfoObject(languageResource.getString("maxsfdistNode"),
				ShipParticular.ABS_MAX_SF_DIST,STRENGTH));
		infoList.add(new InfoObject(languageResource.getString("maxbmNode"),
				ShipParticular.ABS_MAX_BM,STRENGTH));
		infoList.add(new InfoObject(languageResource.getString("maxbmdistNode"),
				ShipParticular.ABS_MAX_BM_DIST,STRENGTH));
	}
	
	private void buildTreeTable(){
		for(InfoObject info: infoList){
			addInfo(info.getCategory(), info);
		}
		update();
	}
	
	/**
	 * Refresh the treetable for new value.
	 */
	public void update(){
		((TreeTableModelAdapter) treetable.getModel()).fireTableDataChanged();
	}
	
	//Handles resize of scrollPane.
    private class ScrollResizeListener extends ComponentAdapter {
      public void componentResized(ComponentEvent e){
        if (treetable.getPreferredSize().width <=
        	scroll.getViewport().getExtentSize().width)
        {
          treetable.setAutoResizeMode(
            JTable.AUTO_RESIZE_ALL_COLUMNS);
        } else {
        	treetable.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        }
      }
    }
}
