//: /be/optiloading/gui/MainMenu.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.gui;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.net.URL;
import java.text.NumberFormat;
import java.util.ResourceBundle;
import java.util.prefs.Preferences;

import javax.swing.ImageIcon;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JFileChooser;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.KeyStroke;
import javax.swing.text.TextAction;

import be.optiloading.cargo.AddFrame;
import be.optiloading.cargo.Cargo;
import be.optiloading.help.HelpViewer;
import be.optiloading.helpers.files.Doc;
import be.optiloading.helpers.files.FileHelper;
import be.optiloading.helpers.files.XmlFilter;
import be.optiloading.optimization.Lp;
import be.optiloading.optimization.Sa;
import be.optiloading.optimization.SectionList;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;
import be.optiloading.tank.Tank;

/**
 * MainMenu class creates the main menu.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class MainMenu extends JMenuBar {
	private static boolean isMac;
	private JMenuItem startLP;
	private JMenuItem startSA;
	private JMenuItem stepSA;
	private JMenuItem resetSA;
	private JMenuItem correctBoth;
	private JMenuItem emptyBallast;
	private JMenuItem emptyCargo;
	private JMenuItem emptyMisc;
	private JMenuItem correctTrim;
	private JMenuItem correctHeel;
	private JMenuItem settings;
	private JMenu opti;
	private JMenuItem newCondition;
	private JMenuItem load;
	private JMenuItem save;
	private JMenuItem saveAs;
	private JMenuItem undo;
	private JMenuItem redo;
	private JMenuItem cut;
	private JMenuItem copy;
	private JMenuItem paste;
	private JCheckBoxMenuItem conventional;
	private ResourceBundle languageResource;
	private Sa StSA;
	private int stepNumber;
	private String stepCargo;
	private RecentMenu recentMenu;
	private Preferences prefs = Preferences.userNodeForPackage(this.getClass());
	
	/**
	 * Constructor which builds the main menu.
	 */
	public MainMenu(boolean conventionalb){
		if (System.getProperty("mrj.version")!=null){
			isMac = true;
		} else {
			isMac = false;
		}
		languageResource = Settings.getInstance().getResourceBundle();
		// File Menu
		JMenu file = new JMenu(languageResource.getString("file"));
		newCondition = new JMenuItem(languageResource.getString("newCon"),
				new ImageIcon(this.getClass().getResource("/images/new.png")));
		newCondition.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(saveCurrent()){
					Doc.getInstance().emptyDoc();
				}
			}
		});
		load = new JMenuItem(languageResource.getString("loadCon"),
				new ImageIcon(this.getClass().getResource("/images/open.png")));
		load.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				openFile(null);
			}
		});
		
		recentMenu = new RecentMenu(languageResource.getString("recentMenu"));
		
		save = new JMenuItem(languageResource.getString("saveCon"),
				new ImageIcon(this.getClass().getResource("/images/save.png")));
		save.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				save();
			}
		});
		save.setEnabled(false);
		saveAs = new JMenuItem(languageResource.getString("saveAsCon"));
		saveAs.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				saveAs();
			}
		});
		settings = new JMenuItem(languageResource.getString("settings"),
				new ImageIcon(this.getClass().getResource("/images/settings.png")));
		settings.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				Settings.getInstance().showDialog();
			}
			
		});
		JMenuItem quit = new JMenuItem(languageResource.getString("quit"),
				new ImageIcon(this.getClass().getResource("/images/quit.png")));
		quit.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				MainFrame.getInstance().saveAndClose();
			}
		});
		file.add(newCondition);
		file.addSeparator();
		file.add(load);
		file.add(recentMenu);
		file.addSeparator();
		file.add(save);
		file.add(saveAs);
		if(isMac == false){
			file.addSeparator();
			file.add(settings);
			file.addSeparator();
			file.add(quit);
		}
		
		// Optimize Menu
		opti = new JMenu(languageResource.getString("optimization"));
		startLP = new JMenuItem(languageResource.getString("startLP"));
		startLP.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(ShipData.getInstance().getCargoList().size() != 0){
					if(!AddFrame.isNull()){
						AddFrame.getInstance().setVisible(false);
					}
					ShipData.getInstance().getBallastList().emptyTanks(false);
					Lp solver = new Lp(Settings.getInstance().getSolver(),false);
					solver.start();
				} else {
					JOptionPane.showMessageDialog(MainFrame.getInstance(), 
							languageResource.getString("noCargo"));
				}	
			}
		});
		startSA = new JMenuItem(languageResource.getString("startSA"));
		startSA.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				if(ShipData.getInstance().getCargoList().size() != 0){
					if(!AddFrame.isNull()){
						AddFrame.getInstance().setVisible(false);
					}
					ShipData.getInstance().getBallastList().emptyTanks(false);
					new Sa(false,false).start();
				} else {
					JOptionPane.showMessageDialog(MainFrame.getInstance(), 
							languageResource.getString("noCargo"));
				}
			}
		});
		stepNumber = 0;
		stepSA = new JMenuItem(languageResource.getString("stepSA")+" (0)");
		stepSA.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				String current = "";
				for(Cargo x:  ShipData.getInstance().getCargoList()){
					current += String.valueOf(x.getVolume())+
						String.valueOf(x.getDensity());
				}
				for(Tank x: ShipData.getInstance().getCargoTankList()){
					current += String.valueOf(x.getMaxfactor());
				}
				if(StSA == null){
					if(ShipData.getInstance().getCargoList().size() != 0){
						ShipData.getInstance().getBallastList().emptyTanks(false);
						StSA = new Sa(false,true);
						StSA.step();
						stepNumber++;
						stepSA.setText(languageResource.getString("stepSA")
									+" ("+String.valueOf(stepNumber)+")");
						stepCargo = current;
						resetSA.setEnabled(true);
					} else {
						JOptionPane.showMessageDialog(MainFrame.getInstance(), 
								languageResource.getString("noCargo"));
					}
				} else {
					if(current.equals(stepCargo)){
						ShipData.getInstance().getBallastList().emptyTanks(false);
						StSA.step();
						stepNumber++;
						stepSA.setText(languageResource.getString("stepSA")
								+" ("+String.valueOf(stepNumber)+")");
					} else {
						int ret = JOptionPane.showConfirmDialog(MainFrame.getInstance(),
								languageResource.getString("cargoChanged"), 
								languageResource.getString("stepReset"),
						        JOptionPane.YES_NO_OPTION,
						        JOptionPane.QUESTION_MESSAGE);
						if(ret == JOptionPane.YES_OPTION){
							ShipData.getInstance().getCargoTankList().emptyTanks(true);
							ShipData.getInstance().getBallastList().emptyTanks(false);
							StSA = new Sa(false,true);
							StSA.step();
							stepCargo = current;
							stepNumber = 1;
							stepSA.setText(languageResource.getString("stepSA")
										+" ("+String.valueOf(stepNumber)+")");
						}
					}
				}
				
			}
		});
		resetSA = new JMenuItem(languageResource.getString("resetSA"));
		resetSA.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				StSA = null;
				resetSA.setEnabled(false);
				ShipData.getInstance().getCargoTankList().emptyTanks(true);
				ShipData.getInstance().getBallastList().emptyTanks(false);
				MainFrame.getInstance().updateInterface();
				stepNumber = 0;
				stepSA.setText(languageResource.getString("stepSA")
							+" ("+String.valueOf(stepNumber)+")");
			}
		});
		resetSA.setEnabled(false);
		
		// Write report
		JMenuItem test = new JMenuItem("Write report");
		test.addActionListener(new ActionListener(){
	        
			public void actionPerformed(ActionEvent e) {
				NumberFormat formatter = NumberFormat.getInstance();
		        formatter.setMaximumFractionDigits(2);
		        formatter.setMinimumFractionDigits(2);
				String[] cargovol = new String[10];
				ShipData data = ShipData.getInstance();
				for(int i = 0; i < 10; i++){
					cargovol[i]= formatter.format(data.getCargoTankList().get(i).getVolume()/
					data.getCargoTankList().get(i).getMax());
				}
				
				String imo = "nee";
		        if((data.getGm()>=0.15f) && (data.getArea30() >= 0.055f) && 
		        		(data.getArea40()>=0.09f) && (data.getArea30to40()>= 0.03f) &&
		        		(data.getMaxGZAngle() > 30f) && (data.getGZ30() >= 0.2f)){
		        	imo = "ja";
		        }
				
		        String trim = formatter.format(data.getTrim());
		        String heel = formatter.format(data.getHeel());
		        
		        SectionList list = new SectionList(data.getDisplacement(), data.getLcb(), 
		        		data.getMctc(), data.getLcf(), data.getTrim());
		        float z = 0f;
		        for(int i = 0; i < 5; i++){
		        	z += Math.abs(data.getCargoTankList().get(2*i).getWeight() +
		        			data.getCargoTankList().get((2*i)+1).getWeight() +
		        			+ list.get(i).getBuoymisc());
		        }
		        formatter.setMaximumFractionDigits(0);
		        formatter.setMinimumFractionDigits(0);
		        String load = formatter.format(z);
		        String ballast = formatter.format(data.getBallastList().getTotalVolumes());
				try {
					FileWriter fwriter = new FileWriter("/Users/Tim/report.txt",true);
					
					fwriter.write("& "+ cargovol[0] + " & " + cargovol[2]  + " & " 
							+ cargovol[4]  + " & " + cargovol[6]  + " & " 
							+ cargovol[8]  + " & " + trim + " & " + heel + " & " +
							imo + " & " + load + " & " + ballast +"\\\\ \n");
					fwriter.write("& "+ cargovol[1] + " & " + cargovol[3]  + " & " 
							+ cargovol[5]  + " & " + cargovol[7]  + " & " 
							+ cargovol[9]  +"&&&&&\\\\ \n");
					fwriter.close();
				} catch (IOException ex) {
					ex.printStackTrace();
				}
				
			}
			
		});
		
		opti.add(startLP);
		opti.addSeparator();
		opti.add(startSA);
		opti.add(stepSA);
		opti.add(resetSA);
		//opti.add(test);
		
		// Corrections
		JMenu corrections = new JMenu(languageResource.getString("corrections"));
		correctTrim = new JMenuItem(languageResource.getString("correctTrim"));
		correctTrim.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().correctTrim(true);
				MainFrame.getInstance().updateInterface();
			}
		});
		correctHeel = new JMenuItem(languageResource.getString("correctHeel"));
		correctHeel.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().correctHeel(true);
				MainFrame.getInstance().updateInterface();
			}
		});
		correctBoth = new JMenuItem(languageResource.getString("correctBoth"));
		correctBoth.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().correctBoth(true);
				MainFrame.getInstance().updateInterface();
			}
		});
		emptyBallast = new JMenuItem(languageResource.getString("emptyBallast"));
		emptyBallast.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().getBallastList().emptyTanks(false);
				MainFrame.getInstance().updateInterface();
			}
		});
		emptyCargo = new JMenuItem(languageResource.getString("emptyCargo"));
		emptyCargo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().getCargoTankList().emptyTanks(true);
				MainFrame.getInstance().updateInterface();
			}
		});
		emptyMisc = new JMenuItem(languageResource.getString("emptyMisc"));
		emptyMisc.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				ShipData.getInstance().getMiscTankList().emptyTanks(true);
				MainFrame.getInstance().updateInterface();
			}
		});
	
		corrections.add(correctTrim);
		corrections.add(correctHeel);
		corrections.add(correctBoth);
		corrections.addSeparator();
		corrections.add(emptyCargo);
		corrections.add(emptyBallast);
		corrections.add(emptyMisc);
		
		
		// Info Menu
		JMenu info = new JMenu(languageResource.getString("info"));
		JMenuItem helpitem = new JMenuItem(languageResource.getString("help"),
				new ImageIcon(this.getClass().getResource("/images/help.png")));
		helpitem.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				HelpViewer.getInstance().setVisible(true);
			}
		});
		
		JMenuItem logItem = new JMenuItem(languageResource.getString("log"));
		logItem.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				Log.getInstance().setVisible(true);
			}
			
		});
		
		JMenuItem about = new JMenuItem(languageResource.getString("about"));
		about.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				new AboutBox();
			}
		});
		
		info.add(helpitem);
		info.add(logItem);
		if(isMac == false){
			info.addSeparator();
			info.add(about);
		}
		
		// View Menu
		JMenu view = new JMenu(languageResource.getString("view"));
		conventional = new JCheckBoxMenuItem(languageResource.getString("conventional"));
		conventional.setSelected(conventionalb);
		if(conventionalb){
			opti.setVisible(false);
		}
		conventional.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(conventional.isSelected()){
					MainFrame.getInstance().toConventional(false);
					MainFrame.getInstance().updateInterface();
					opti.setVisible(false);
				}else {
					MainFrame.getInstance().toAutomatic(false);
					MainFrame.getInstance().updateInterface();
					opti.setVisible(true);
				}
				
			}
		});
		view.add(conventional);
		
		// Edit menu
		JMenu edit = new JMenu(languageResource.getString("edit"));
		undo = new JMenuItem(languageResource.getString("undo"),
				new ImageIcon(this.getClass().getResource("/images/undo.png")));
		undo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				Doc.getInstance().undo();
				update();
			}
		});
		redo = new JMenuItem(languageResource.getString("redo"),
				new ImageIcon(this.getClass().getResource("/images/redo.png")));
		redo.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				Doc.getInstance().redo();
				update();
			}
		});
		cut = new JMenuItem(languageResource.getString("cut"),
				new ImageIcon(this.getClass().getResource("/images/cut.png")));
		cut.addActionListener(new TextAction("cut") {
			public void actionPerformed(ActionEvent e) {
				Object o = getFocusedComponent();
             	if(o==null){
             		return;
             	} else {
             		getFocusedComponent().cut();
             	}
			}
		});
		copy = new JMenuItem(languageResource.getString("copy"),
				new ImageIcon(this.getClass().getResource("/images/copy.png")));
		copy.addActionListener(new TextAction("copy") {
			public void actionPerformed(ActionEvent e) {
				Object o = getFocusedComponent();
             	if(o==null){
             		return;
             	} else {
             		getFocusedComponent().copy();
             	}
			}
		});
		paste = new JMenuItem(languageResource.getString("paste"),
				new ImageIcon(this.getClass().getResource("/images/paste.png")));
		paste.addActionListener(new TextAction("copy") {
			public void actionPerformed(ActionEvent e) {
				Object o = getFocusedComponent();
             	if(o==null){
             		return;
             	} else {
             		getFocusedComponent().paste();
             	}
			}
		});
		edit.add(undo);
		edit.add(redo);
		edit.addSeparator();
		edit.add(cut);
		edit.add(copy);
		edit.add(paste);
		
		// Ad to main menu bar
		add(file);
		add(edit);
		add(view);
		add(opti);
		add(corrections);
		add(info);
		
		newCondition.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_N, ActionEvent.CTRL_MASK));
		load.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_O, ActionEvent.CTRL_MASK));
		save.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_S, ActionEvent.CTRL_MASK));
		saveAs.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_S, ActionEvent.SHIFT_MASK
		        | ActionEvent.CTRL_MASK));
		quit.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
		conventional.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_C, ActionEvent.SHIFT_MASK
		        | ActionEvent.CTRL_MASK));
		undo.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Z, ActionEvent.CTRL_MASK));
		redo.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_Z, ActionEvent.SHIFT_MASK
		        | ActionEvent.CTRL_MASK));
		copy.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_C, ActionEvent.CTRL_MASK));
		paste.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_V, ActionEvent.CTRL_MASK));
		cut.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_X, ActionEvent.CTRL_MASK));
		startLP.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_L, ActionEvent.CTRL_MASK));
		startSA.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_A, ActionEvent.CTRL_MASK));
		stepSA.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_A, ActionEvent.SHIFT_MASK
		        | ActionEvent.CTRL_MASK));
		resetSA.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_R, ActionEvent.CTRL_MASK));
		correctTrim.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_T, ActionEvent.CTRL_MASK));
		correctHeel.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_H, ActionEvent.CTRL_MASK));
		correctBoth.setAccelerator(KeyStroke.getKeyStroke(
		        KeyEvent.VK_B, ActionEvent.CTRL_MASK));
		helpitem.setAccelerator(KeyStroke.getKeyStroke("F1"));
		
		if(Settings.getInstance().getLanguage() == 0){
			file.setMnemonic(KeyEvent.VK_F);
			newCondition.setMnemonic(KeyEvent.VK_N);
			load.setMnemonic(KeyEvent.VK_O);
			recentMenu.setMnemonic(KeyEvent.VK_R);
			save.setMnemonic(KeyEvent.VK_S);
			saveAs.setMnemonic(KeyEvent.VK_A);
			settings.setMnemonic(KeyEvent.VK_P);
			quit.setMnemonic(KeyEvent.VK_Q);

			view.setMnemonic(KeyEvent.VK_V);
			conventional.setMnemonic(KeyEvent.VK_C);
			
			edit.setMnemonic(KeyEvent.VK_E);
			undo.setMnemonic(KeyEvent.VK_U);
			redo.setMnemonic(KeyEvent.VK_R);
			cut.setMnemonic(KeyEvent.VK_T);
			copy.setMnemonic(KeyEvent.VK_C);
			paste.setMnemonic(KeyEvent.VK_P);

			opti.setMnemonic(KeyEvent.VK_O);
			startLP.setMnemonic(KeyEvent.VK_L);
			startSA.setMnemonic(KeyEvent.VK_S);
			stepSA.setMnemonic(KeyEvent.VK_A);
			resetSA.setMnemonic(KeyEvent.VK_R);

			corrections.setMnemonic(KeyEvent.VK_C);
			correctTrim.setMnemonic(KeyEvent.VK_T);
			correctHeel.setMnemonic(KeyEvent.VK_H);
			correctBoth.setMnemonic(KeyEvent.VK_O);
			emptyBallast.setMnemonic(KeyEvent.VK_B);
			emptyCargo.setMnemonic(KeyEvent.VK_T);
			emptyMisc.setMnemonic(KeyEvent.VK_M);
			
			info.setMnemonic(KeyEvent.VK_I);
			helpitem.setMnemonic(KeyEvent.VK_H);
			logItem.setMnemonic(KeyEvent.VK_L);
			about.setMnemonic(KeyEvent.VK_A);
		} else {
			file.setMnemonic(KeyEvent.VK_B);
			newCondition.setMnemonic(KeyEvent.VK_N);
			load.setMnemonic(KeyEvent.VK_O);
			recentMenu.setMnemonic(KeyEvent.VK_R);
			save.setMnemonic(KeyEvent.VK_S);
			saveAs.setMnemonic(KeyEvent.VK_A);
			settings.setMnemonic(KeyEvent.VK_V);
			quit.setMnemonic(KeyEvent.VK_A);

			view.setMnemonic(KeyEvent.VK_D);
			conventional.setMnemonic(KeyEvent.VK_C);
			
			edit.setMnemonic(KeyEvent.VK_W);
			undo.setMnemonic(KeyEvent.VK_O);
			redo.setMnemonic(KeyEvent.VK_H);
			cut.setMnemonic(KeyEvent.VK_N);
			copy.setMnemonic(KeyEvent.VK_K);
			paste.setMnemonic(KeyEvent.VK_P);

			opti.setMnemonic(KeyEvent.VK_O);
			startLP.setMnemonic(KeyEvent.VK_L);
			startSA.setMnemonic(KeyEvent.VK_S);
			stepSA.setMnemonic(KeyEvent.VK_A);
			resetSA.setMnemonic(KeyEvent.VK_R);

			corrections.setMnemonic(KeyEvent.VK_C);
			correctTrim.setMnemonic(KeyEvent.VK_T);
			correctHeel.setMnemonic(KeyEvent.VK_H);
			correctBoth.setMnemonic(KeyEvent.VK_E);
			emptyBallast.setMnemonic(KeyEvent.VK_B);
			emptyCargo.setMnemonic(KeyEvent.VK_L);
			emptyMisc.setMnemonic(KeyEvent.VK_O);
			
			info.setMnemonic(KeyEvent.VK_I);
			helpitem.setMnemonic(KeyEvent.VK_H);
			logItem.setMnemonic(KeyEvent.VK_L);
			about.setMnemonic(KeyEvent.VK_O);
		}
	}
	
	/**
	 * Lock part of main menu while performing algorithm.
	 */
	public void lock(){
		startLP.setEnabled(false);
		settings.setEnabled(false);
		load.setEnabled(false);
		recentMenu.setEnabled(false);
		newCondition.setEnabled(false);
		save.setEnabled(false);
		saveAs.setEnabled(false);
		startSA.setEnabled(false);
		stepSA.setEnabled(false);
		resetSA.setEnabled(false);
		correctBoth.setEnabled(false);
		emptyBallast.setEnabled(false);
		emptyCargo.setEnabled(false);
		emptyMisc.setEnabled(false);
		correctTrim.setEnabled(false);
		correctHeel.setEnabled(false);
		conventional.setEnabled(false);
		undo.setEnabled(false);
		redo.setEnabled(false);
		cut.setEnabled(false);
		copy.setEnabled(false);
		paste.setEnabled(false);
	}
	
	/**
	 * Unlock main menu.
	 */
	public void unlock(){
		startLP.setEnabled(true);
		settings.setEnabled(true);
		newCondition.setEnabled(true);
		saveAs.setEnabled(true);
		load.setEnabled(true);
		recentMenu.setEnabled(true);
		startSA.setEnabled(true);
		stepSA.setEnabled(true);
		if(StSA != null){
			resetSA.setEnabled(true);
		}
		correctBoth.setEnabled(true);
		correctTrim.setEnabled(true);
		correctHeel.setEnabled(true);
		conventional.setEnabled(true);
		emptyBallast.setEnabled(true);
		emptyCargo.setEnabled(true);
		emptyMisc.setEnabled(true);
		cut.setEnabled(true);
		copy.setEnabled(true);
		paste.setEnabled(true);
		update();
	}
	
	/**
	 * Update the main menu.
	 */
	public void update(){
		save.setEnabled(Doc.getInstance().isSave());
		redo.setEnabled(Doc.getInstance().canRedo());
		undo.setEnabled(Doc.getInstance().canUndo());
	}
	
	/**
	 * Procedure to save file.
	 * @return Returns true if succesfully saved.
	 */
	public boolean save(){
		boolean returnvalue = true;
		if(Doc.getInstance().getCurrentFile() != null){
			Doc.getInstance().save();
		} else {
			returnvalue = saveAs();
		}
		return returnvalue;
	}
	
	/**
	 * Procedure to save as
	 * @return Returns true if succesfully saved.
	 */
	public boolean saveAs(){
		boolean returnvalue = true;
		File file = FileHelper.getValidFile(".xml", new XmlFilter());
		if(file != null){
			Doc.getInstance().saveAs(file);
		} else {
			returnvalue = false;
		}
		return returnvalue;
	}
	
	/**
	 * Method to check if file changed and open file.
	 * @param openfile File object to open.
	 */
	public void openFile(File openfile){
		if(saveCurrent()){
			if(openfile == null){
			JFileChooser fc = new JFileChooser();
			fc.setCurrentDirectory(new File(prefs.get("currentDir", 
					fc.getCurrentDirectory().getPath())));
			fc.setFileSelectionMode(JFileChooser.FILES_ONLY);
			fc.setMultiSelectionEnabled(false);
			fc.setFileFilter(new XmlFilter());
			int returnVal = fc.showOpenDialog(MainFrame.getInstance());
			if (returnVal == JFileChooser.APPROVE_OPTION){
				File file = fc.getSelectedFile();
				URL xsdURL = this.getClass().getResource("/data/file.xsd");
				if(FileHelper.checkFile(file,null,xsdURL)){
					Doc.getInstance().load(file);
				} else {
					JOptionPane.showMessageDialog(MainFrame.getInstance(), 
							Settings.getInstance().getResourceBundle()
								.getString("invalidFile"),
							Settings.getInstance().getResourceBundle()
								.getString("exceptionTitle"),
							JOptionPane.WARNING_MESSAGE);
				}
			}
			prefs.put("currentDir", fc.getCurrentDirectory().getPath());
			} else {
				Doc.getInstance().load(openfile);
			}
		}
	}
	
	// Save current condition
	private boolean saveCurrent(){
		boolean returnvalue = true;
		if(Doc.getInstance().isCanged()){
			int ret = JOptionPane.showConfirmDialog(MainFrame.getInstance(),
					languageResource.getString("saveFile"), 
					languageResource.getString("saveChanges"),
			        JOptionPane.YES_NO_CANCEL_OPTION,
			        JOptionPane.QUESTION_MESSAGE);
			if (ret == JOptionPane.YES_OPTION){
				returnvalue = save();
			} else if(ret == JOptionPane.CANCEL_OPTION){
				returnvalue = false;
			}
		}
		return returnvalue;
	}

	/**
	 * Returns the recently opened condition menu.
	 * @return RecentMenu with recently opened files.
	 */
	public RecentMenu getRecentMenu() {
		return recentMenu;
	}
}
