//: be/optiloading/gui/Splash.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.gui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Font;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.ResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JProgressBar;
import javax.swing.JWindow;
import javax.swing.UIManager;
import javax.swing.border.EmptyBorder;

import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * Splash class created the splash screen. This is the main class.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class Splash extends JWindow {
	private JLabel progress;
	private JProgressBar progressbar;
	private ResourceBundle languageResource;
	
	/**
	 * Main method to start program
	 * @param args Parameters read from command line.
	 */
	public static void main(String[] args){
		if (System.getProperty("mrj.version")!=null){
			System.setProperty("com.apple.mrj.application.apple.menu.about.name", "OptiLoading");
			System.setProperty("apple.laf.useScreenMenuBar","true");
			try {
				Class<?> osxAdapter = ClassLoader.getSystemClassLoader().
				loadClass("be.optiloading.helpers.osspecific.OsXAdapter");
				Constructor<?> new_adapter = osxAdapter.getConstructor();
				new_adapter.newInstance();
			}
			catch(Exception e) {
				e.printStackTrace();
				Log.getInstance().add(Log.JAVA,e);
			}
		}
		new Splash();
	}
	
	/**
	 * Constructor.
	 */
	public Splash() {
		URL iconURL = this.getClass().getResource("/images/splash.png");
		ImageIcon icon = new ImageIcon(iconURL);
		JLabel image = new JLabel(icon);
		
		MouseAdapter clickListener = new MouseAdapter(){
			public void mouseClicked(MouseEvent e) {
				closeSplash();
			}
		};
		
		Settings.getInstance();
		languageResource = Settings.getInstance().getResourceBundle();
		
		UIManager.put("OptionPane.yesButtonText", 
				languageResource.getString("Yes"));
		UIManager.put("OptionPane.noButtonText", 
				languageResource.getString("No"));
		UIManager.put("OptionPane.cancelButtonText", 
				languageResource.getString("Cancel"));
		
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadSettings"));
		progress = new JLabel(languageResource.getString("loadSettings"));
		progress.setOpaque(false);
		progress.setBorder(new EmptyBorder(0,3,2,0));
		progress.setFont(new Font("Dialog",Font.PLAIN,10));
		progress.setForeground(new Color(18,65,138));
		JPanel progresspanel = new JPanel(new BorderLayout());
		progressbar = new JProgressBar();
		progressbar.setMinimum(0);
		progressbar.setMaximum(10);
		progresspanel.add(BorderLayout.NORTH,progress);
		progresspanel.add(BorderLayout.SOUTH, progressbar);
		progresspanel.setOpaque(false);
		
		
		JPanel content = (JPanel) getContentPane();
		getLayeredPane().add(image,new Integer(Integer.MIN_VALUE));
		image.setBounds(0, 0, icon.getIconWidth(), icon.getIconHeight());
		content.setOpaque(false);
		content.add(BorderLayout.SOUTH, progresspanel);

		setSize(332, 208);
		setLocationRelativeTo(null);
		setVisible(true);

		long starttime = System.currentTimeMillis();
		loadProgram();
		
		// Once the program is loaded the user can click the splash screen to terminate
		image.addMouseListener(clickListener);
		
		
		// Wait two seconds before terminating
		long currenttime=System.currentTimeMillis();
	    while( ((starttime+2000) >currenttime)){
	          currenttime=System.currentTimeMillis();	          
	    }
	    closeSplash();
	}
	
	// Terminate the splash screen
	private void closeSplash(){
		this.dispose();
		MainFrame.getInstance().setVisible(true);
		MainFrame.getInstance().setAlwaysOnTop(false);
		MainFrame.getInstance().updateInterface();
		Log.getInstance().setStdout();
	}
	
	// Load the GUI and ship object while the splash screen is showing
	private void loadProgram() {
		progress.setText(languageResource.getString("loadShipData"));
		progressbar.setValue(0);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadShipData"));
		ShipData data = ShipData.getInstance();
		data.setSplash(this);
		
		try{
			data.initializeShipData("/data/ship.xml");
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadStabilityData"));
		progressbar.setValue(1);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadStabilityData"));
		try{
			data.initializeStabilityData("/data/hydrostatic.xml", "/data/kn.xml");
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadStrengthData"));
		progressbar.setValue(2);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadStrengthData"));
		try{
			data.initializeStrengthData("/data/lightship.xml", "/data/buoyancy.xml");
		} catch (Exception e){
			reportException(e);
		}
			
		progress.setText(languageResource.getString("loadCargoTankData"));
		progressbar.setValue(3);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadCargoTankData"));
		try{
			data.initializeCargoTankList("/data/cargotanks.xml");
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadBallastData"));
		progressbar.setValue(4);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadBallastData"));
		try{
			data.initializeBallastTankList("/data/ballasttanks.xml");
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadMiscTankData"));
		progressbar.setValue(5);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadMiscTankData"));
		try{
			data.initializeMiscTankList("/data/misctanks.xml");
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadCargoList"));
		progressbar.setValue(6);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadCargoList"));
		try{
			data.initializeCargoList();
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadCargoCompList"));
		progressbar.setValue(7);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadCargoCompList"));
		try{
			data.initializeCargoCompList();
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadTankCompList"));
		progressbar.setValue(8);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadTankCompList"));
		try{
			data.initializeTankCompList();
		} catch (Exception e){
			reportException(e);
		}
		
		progress.setText(languageResource.getString("loadInterface"));
		progressbar.setValue(9);
		Log.getInstance().add(Log.OPTILOADING,
				languageResource.getString("loadInterface"));
		try{
			MainFrame.getInstance();
		} catch (Exception e){
			reportException(e);
		}

		progress.setText(languageResource.getString("messageDone"));
		progressbar.setValue(10);
		
		data.setSplash(null);
		data.getBallastList().emptyTanks(false);
		data = null;
	}
	
	// Show that something went wrong
	private void reportException(Exception e){
		String osName = System.getProperty("os.name");
		if(osName.equals("Linux")){
			toBack();
		}
		Log.getInstance().add(Log.JAVA,e);
		JOptionPane.showMessageDialog(this, 
				languageResource.getString("exceptionMessage"),
				languageResource.getString("exceptionTitle"),
				JOptionPane.WARNING_MESSAGE);
	}
}
