//: be/optiloading/help/HelpViewer.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.help;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.util.ArrayList;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextField;
import javax.swing.JToolBar;
import javax.swing.JTree;
import javax.swing.border.EmptyBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.Document;
import javax.swing.text.EditorKit;
import javax.swing.text.html.HTMLDocument;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.text.html.HTMLFrameHyperlinkEvent;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import be.optiloading.gui.Log;

/**
 * HelpViewer class is used to create the viewer for the help information.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class HelpViewer extends JFrame {
	private static final HelpInfo searchtopnode = new HelpInfo("Search results",
			"/help/searchresults.html");
	private JEditorPane htmlPane;
	private HelpTree tree;
	private JTree searchtree;
	private DefaultMutableTreeNode searchtop;
	private DefaultTreeModel model;
	private JTextField search;
	private JButton back;
	private JButton forward;
	private JPanel cards;
	private JPanel searchpanel;
	private JPanel treepanel;
	private JScrollPane treeView;

	// Holds the previous viewed pages to enabled the back/forward buttons
	private ArrayList<URL> history = new ArrayList<URL>();

	// Number indicating how many places in the history the user went back
	private int backvalue = 0;
	private boolean hyperlink = false;
	
	private static HelpViewer uniqueInstance;
	
	/**
	 * Get singleton instance.
	 * 
	 * @return Log object.
	 */
	public static HelpViewer getInstance(){
		if (uniqueInstance == null) {
			uniqueInstance = new HelpViewer();
		}
		return uniqueInstance;
	}
	
	private HelpViewer() {
		// Create main panel
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());

		// Create the tree
		tree = new HelpTree();
		tree.addTreeSelectionListener(new TreeListener());
		model = (DefaultTreeModel) tree.getModel();

		// Create the scroll pane and add the tree to it
		treeView = new JScrollPane(tree);
		treepanel = new JPanel();
		treepanel.setLayout(new BorderLayout());
		treepanel.add(BorderLayout.CENTER,treeView);
		
		// Search tree
		searchtop = new DefaultMutableTreeNode(searchtopnode);
		searchtree = new JTree(searchtop);
		searchtree.getSelectionModel().setSelectionMode(
				TreeSelectionModel.SINGLE_TREE_SELECTION);
		DefaultTreeCellRenderer renderer = new DefaultTreeCellRenderer();
		renderer.setOpenIcon(new ImageIcon(this.getClass()
				.getResource("/images/opened.gif")));
		renderer.setClosedIcon(new ImageIcon(this.getClass()
				.getResource("/images/closed.gif")));
		renderer.setLeafIcon(new ImageIcon(this.getClass()
				.getResource("/images/leaf.gif")));
		searchtree.setCellRenderer(renderer);
		searchtree.addTreeSelectionListener(new SearchTreeListener());
		JScrollPane searchtreeView = new JScrollPane(searchtree);

		// Close search panel
		JPanel closesearch = new JPanel();
		closesearch.setLayout(new BorderLayout());
		JButton close = new JButton(new ImageIcon(HelpViewer.class
				.getResource("/images/close.png")));
		close.setToolTipText("Close search results.");
		close.addActionListener(new CloseListener());
		closesearch.add(BorderLayout.EAST, close);

		// Search panel
		searchpanel = new JPanel();
		searchpanel.setLayout(new BorderLayout());
		searchpanel.add(BorderLayout.NORTH, closesearch);
		searchpanel.add(BorderLayout.CENTER, searchtreeView);

		// Card Layout
		cards = new JPanel(new CardLayout());
		cards.add(treepanel, "TREE");
		cards.add(searchpanel, "SEARCH");

		// Create the HTML viewing pane
		htmlPane = new JEditorPane();
		htmlPane.setEditable(false);
		htmlPane.addHyperlinkListener(new Hyperactive());
		JScrollPane htmlView = new JScrollPane(htmlPane);

		// Add the scroll panes to a split pane
		JSplitPane splitPane = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
				cards, htmlView);
		splitPane.setOneTouchExpandable(true);

		htmlView.setPreferredSize(new Dimension(400, 550));
		cards.setPreferredSize(new Dimension(250, 550));
		splitPane.setDividerLocation(200);
		splitPane.setPreferredSize(new Dimension(650, 550));

		// Add the split pane to the panel
		panel.add(BorderLayout.CENTER, splitPane);

		// Create the toolbar
		JToolBar toolbar = new JToolBar();
		toolbar.setFloatable(false);
		toolbar.setRollover(true);
		back = new JButton(new ImageIcon(HelpViewer.class
				.getResource("/images/previous.png")));
		back.setToolTipText("Go to previous page");
		forward = new JButton(new ImageIcon(HelpViewer.class
				.getResource("/images/next.png")));
		forward.setToolTipText("Go to following page");
		JButton home = new JButton(new ImageIcon(HelpViewer.class
				.getResource("/images/home.png")));
		home.setToolTipText("Go home");
		back.setEnabled(false);
		back.addActionListener(new BackListener());
		forward.setEnabled(false);
		forward.addActionListener(new ForwardListener());
		home.addActionListener(new HomeListener());
		toolbar.add(back);
		toolbar.add(forward);
		toolbar.add(home);

		// Search field
		JPanel searchfieldpanel = new JPanel();
		searchfieldpanel.setLayout(new BoxLayout(searchfieldpanel,
				BoxLayout.Y_AXIS));
		search = new JTextField();
		search.setPreferredSize(new Dimension(100, 20));
		search.setMinimumSize(new Dimension(100, 20));
		search.setMaximumSize(new Dimension(100, 20));
		search.setText("search");
		search.addFocusListener(new SearchFocusListener());
		search.addActionListener(new SearchListener());
		searchfieldpanel.setBorder(new EmptyBorder(8, 0, 8, 8));
		searchfieldpanel.add(BorderLayout.CENTER, search);
		JPanel toolbarpanel = new JPanel();
		toolbarpanel.setLayout(new BorderLayout());
		toolbarpanel.add(BorderLayout.WEST, toolbar);
		toolbarpanel.add(BorderLayout.EAST, searchfieldpanel);
		panel.add(BorderLayout.PAGE_START, toolbarpanel);

		// Configure the frame
		setTitle("OptiLoading Help");
		home.doClick();
		getContentPane().add(panel);
		setIconImage(new ImageIcon(this.getClass().getResource(
				"/images/icon.png")).getImage());
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
		
		addWindowListener(new WindowAdapter(){
			public void windowClosing(WindowEvent arg0) {
				uniqueInstance = null;
			}
		});

	}

	// Selects the text in the search fields when the focus is gained.
	private class SearchFocusListener implements FocusListener {
		public void focusGained(FocusEvent e) {
			JTextField source = (JTextField) e.getSource();
			source.selectAll();
		}

		public void focusLost(FocusEvent e) {
		}
	}

	// Performed when a search action is requested.
	private class SearchListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			searchtop = new DefaultMutableTreeNode(searchtopnode);
			searchtree.setModel(new DefaultTreeModel(searchtop));
			DefaultTreeModel searchmodel = (DefaultTreeModel) searchtree.getModel();
			JTextField source = (JTextField) e.getSource();
			searchHelp((DefaultMutableTreeNode) model.getRoot(), source
					.getText().toLowerCase());
			if (searchmodel.getChildCount(searchtop) != 0) {
				DefaultMutableTreeNode node = (DefaultMutableTreeNode) searchmodel
						.getChild(searchtop, 0);
				TreeNode[] path = searchmodel.getPathToRoot(node);
				TreePath treePath = new TreePath(path);
				searchtree.scrollPathToVisible(treePath);
				searchtree.setSelectionPath(treePath);
				searchtree.expandPath(treePath);
				hyperlink = true;
				Object nodeInfo = node.getUserObject();
				HelpInfo help = (HelpInfo) nodeInfo;
				searchAndExpand(help.helpURL);
				hyperlink = false;
			}
			CardLayout cl = (CardLayout)(cards.getLayout());
		    cl.show(cards, "SEARCH");
		}
	}

	// When the close button of the search panel is pressed.
	private class CloseListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			CardLayout cl = (CardLayout)(cards.getLayout());
		    cl.show(cards, "TREE");
		}
	}

	// ActionListener for the back button
	private class BackListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			backvalue++;
			displayURL(history.size() - (1 + backvalue));
			hyperlink = true;
			searchAndExpand(history.get(history.size() - (1 + backvalue)));
			hyperlink = false;
		}
	}

	// ActionLitener for the forward
	private class ForwardListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			backvalue--;
			displayURL(history.size() - (1 + backvalue));
			hyperlink = true;
			searchAndExpand(history.get(history.size() - (1 + backvalue)));
			hyperlink = false;
		}
	}

	// ActionListener for the home button
	private class HomeListener implements ActionListener {
		public void actionPerformed(ActionEvent e) {
			if ((htmlPane.getPage() == null)
					|| (htmlPane.getPage().equals(
							this.getClass().getResource("/help/preface.html")) == false)) {
				history.add(this.getClass().getResource("/help/preface.html"));
				displayURL(history.size() - 1);
				hyperlink = true;
				searchAndExpand(this.getClass()
						.getResource("/help/preface.html"));
				hyperlink = false;
			}
		}
	}

	// TreeSelectionListener to listen to changes in tree selection
	private class TreeListener implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent e) {

			DefaultMutableTreeNode node = (DefaultMutableTreeNode) tree
					.getLastSelectedPathComponent();

			if (node == null)
				return;

			Object nodeInfo = node.getUserObject();
			HelpInfo help = (HelpInfo) nodeInfo;

			// Hyperlink is true when the search and expand is called (URL
			// should only be added once to history)
			if (hyperlink == false) {
				if (backvalue != 0) {
					for (int i = 0; i < backvalue; i++) {
						history.remove(history.size() - 1);
					}
					backvalue = 0;
				}
				history.add(help.helpURL);
				displayURL(history.size() - 1);
			}
		}
	}

	// TreeSelectionListener to listen to changes in tree selection
	private class SearchTreeListener implements TreeSelectionListener {
		public void valueChanged(TreeSelectionEvent e) {

			DefaultMutableTreeNode node = (DefaultMutableTreeNode) searchtree
					.getLastSelectedPathComponent();

			if (node == null)
				return;

			Object nodeInfo = node.getUserObject();
			HelpInfo help = (HelpInfo) nodeInfo;

			// Hyperlink is true when the search and expand is called (URL
			// should only be added once to history)
			if (hyperlink == false) {
				if (backvalue != 0) {
					for (int i = 0; i < backvalue; i++) {
						history.remove(history.size() - 1);
					}
					backvalue = 0;
				}
				history.add(help.helpURL);
				displayURL(history.size() - 1);
			}
			hyperlink = true;
			searchAndExpand(help.helpURL);
			hyperlink = false;
		}
	}

	// HyperlinkListener listens to hyperlinks from htmlPane
	class Hyperactive implements HyperlinkListener {

		public void hyperlinkUpdate(HyperlinkEvent e) {
			if (e.getEventType() == HyperlinkEvent.EventType.ACTIVATED) {
				JEditorPane pane = (JEditorPane) e.getSource();
				if (e instanceof HTMLFrameHyperlinkEvent) {
					HTMLFrameHyperlinkEvent evt = (HTMLFrameHyperlinkEvent) e;
					HTMLDocument doc = (HTMLDocument) pane.getDocument();
					doc.processHTMLFrameHyperlinkEvent(evt);
				} else {
					try {
						if (backvalue != 0) {
							for (int i = 0; i < backvalue; i++) {
								history.remove(history.size() - 1);
							}
							backvalue = 0;
						}
						history.add(e.getURL());
						displayURL(history.size() - 1);
						hyperlink = true;
						searchAndExpand(e.getURL());
						hyperlink = false;
					} catch (Exception t) {
						t.printStackTrace();
						Log.getInstance().add(Log.JAVA,t);
					}
				}
			}
		}
	}

	// Looks for the node depending on the given URL
	private void searchAndExpand(URL helpURL) {
		TreeNode[] path = search((DefaultMutableTreeNode) model.getRoot(),
				helpURL);

		if (path != null) {
			TreePath treePath = new TreePath(path);
			tree.scrollPathToVisible(treePath);
			tree.setSelectionPath(treePath);
			tree.expandPath(treePath);
		}
	}

	// Search all nodes
	private TreeNode[] search(DefaultMutableTreeNode node, URL helpURL) {
		TreeNode[] path = null;

		Object nodeInfo = node.getUserObject();
		HelpInfo help = (HelpInfo) nodeInfo;

		if (helpURL.equals(help.helpURL)) {
			path = model.getPathToRoot(node);
		} else {
			int i = 0;
			int n = model.getChildCount(node);

			while ((i < n) && (path == null)) {
				path = search((DefaultMutableTreeNode) model.getChild(node, i),
						helpURL);
				i++;
			}
		}

		return path;
	}

	// Search the help library.
	private void searchHelp(DefaultMutableTreeNode node, String s) {
		Object nodeInfo = node.getUserObject();
		HelpInfo help = (HelpInfo) nodeInfo;
		String html = null;

		EditorKit kit = new HTMLEditorKit();
		Document doc = kit.createDefaultDocument();
		doc.putProperty("IgnoreCharsetDirective", Boolean.TRUE);
		try {
			InputStream stream = help.helpURL.openStream();
			InputStreamReader rd = new InputStreamReader(stream);
			kit.read(rd, doc, 0);
			html = doc.getText(0, doc.getLength());
		} catch (IOException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
		} catch (BadLocationException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
		}

		html = html.toLowerCase();

		if (html.indexOf(s) != -1) {
			DefaultTreeModel searchmodel = (DefaultTreeModel) searchtree.getModel();
			searchmodel.insertNodeInto(new DefaultMutableTreeNode(help),
					searchtop, model.getChildCount(searchtop));
		}
		int i = 0;
		int n = model.getChildCount(node);

		while (i < n) {
			searchHelp((DefaultMutableTreeNode) model.getChild(node, i), s);
			i++;
		}
	}

	// Display the URL in the htmlPane
	private void displayURL(int i) {
		try {
			htmlPane.setPage(history.get(i));
			updateButtons();

		} catch (IOException e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
		}
	}

	// Update the back/forward buttons
	public void updateButtons() {
		if (backvalue > 0) {
			forward.setEnabled(true);
		} else {
			forward.setEnabled(false);
		}
		if ((history.size() > 1) && (backvalue < (history.size() - 1))) {
			back.setEnabled(true);
		} else {
			back.setEnabled(false);
		}
	}
}