//: /be/optiloading/helpers/files/Doc.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.helpers.files;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.ResourceBundle;

import javax.swing.JOptionPane;
import javax.swing.event.UndoableEditEvent;
import javax.swing.event.UndoableEditListener;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;
import javax.swing.undo.UndoManager;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;
import org.xml.sax.helpers.DefaultHandler;

import be.optiloading.cargo.Cargo;
import be.optiloading.cargo.CargoList;
import be.optiloading.compatibility.CargoComp;
import be.optiloading.compatibility.TankComp;
import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;
import be.optiloading.tank.Tank;

/**
 * OptiLoading document object.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class Doc extends PlainDocument{
	protected UndoManager undo = new UndoManager();
	private boolean changed;
	private File currentFile;
	private ResourceBundle languageResource = Settings.getInstance().getResourceBundle();
	private static Doc uniqueInstance;
	
	/**
	 * Returns the uniqueInstance of the Doc object.
	 * 
	 * @return Doc object.
	 */
	public static Doc getInstance() {
		if (uniqueInstance == null) {
			uniqueInstance = new Doc();
		}
		return uniqueInstance;
	}
	
	// Private constructor
	private Doc(){
		replace(getCurrent());
		changed = false;
		currentFile = null;
		addUndoableEditListener(new MyUndoableEditListener());
	}

	/**
	 * Load file into program.
	 * @param file File object that has to be loaded.
	 */
	public void load(File file){
		try {
			BufferedReader in = new BufferedReader(new FileReader(file));
	        String s = "";
	        String str;
	        while ((str = in.readLine()) != null) {
	            s += str.trim();
	        }
	        in.close();
			replace(s);
		} catch (Exception e) {
			e.printStackTrace();
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(MainFrame.getInstance(), 
				Settings.getInstance().getResourceBundle().getString("exceptionMessage"),
				Settings.getInstance().getResourceBundle().getString("exceptionTitle"),
				JOptionPane.WARNING_MESSAGE);
		}
		show();
		currentFile = file;
		changed = false;
		undo.discardAllEdits();
		MainFrame.getInstance().setFileName(file);
		MainFrame.getInstance().updateMenu();
	}
	
	/**
	 * Load light ship condition.
	 */
	public void emptyDoc(){
		ShipData.getInstance().getCargoTankList().emptyTanks(true);
		ShipData.getInstance().getBallastList().emptyTanks(false);
		ShipData.getInstance().getMiscTankList().emptyTanks(true);
		ShipData.getInstance().getCargoTankList().setDefaultMaxFactor(0.98f);
		ShipData.getInstance().getBallastList().setDefaultMaxFactor(1f);
		ShipData.getInstance().getMiscTankList().setDefaultMaxFactor(1f);
		ShipData.getInstance().getCargoCompList().clear();
		ShipData.getInstance().getTankCompList().clear();
		ShipData.getInstance().getCargoList().clear();
		MainFrame.getInstance().setFileName(null);
		MainFrame.getInstance().updateInterface();
		undo.discardAllEdits();
		changed = false;
		currentFile = null;
		MainFrame.getInstance().updateMenu();
	}
	
	// Get the current condition as a string.
	private String getCurrent(){
		String xmlString = null;
		try{
			DocumentBuilderFactory documentBuilderFactory = 
				DocumentBuilderFactory.newInstance();
			DocumentBuilder documentBuilder = 
				documentBuilderFactory.newDocumentBuilder();
			Document document = documentBuilder.newDocument();
			Element rootElement = document.createElement("optiloading");
			document.appendChild(rootElement);
			
			//Cargo List
			Element mainElement = document.createElement("cargo");
			rootElement.appendChild(mainElement);
			for (Cargo cargo: ShipData.getInstance().getCargoList()){
				Element subElement = document.createElement("item");
				mainElement.appendChild(subElement);
				Element em = document.createElement("cargoid");
				String value = cargo.getCargoid();
				em.appendChild(document.createTextNode(value));
				subElement.appendChild(em);
				em = document.createElement("description");
				value = cargo.getDescription();
				em.appendChild(document.createTextNode(value));
				subElement.appendChild(em);
				em = document.createElement("weight");
				float fvalue = cargo.getWeight();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("density");
				fvalue = cargo.getDensity();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
			}
			//Cargo Tanks
			mainElement = document.createElement("cargotanks");
			rootElement.appendChild(mainElement);
			for (Tank tank: ShipData.getInstance().getCargoTankList()){
				Element subElement = document.createElement("item");
				mainElement.appendChild(subElement);
				Element em = document.createElement("volume");
				float fvalue = tank.getVolume();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("density");
				fvalue = tank.getDensity();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("maxpercentage");
				fvalue = tank.getMaxfactor();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				if(tank.getCargoid() != null){
					em = document.createElement("cargoid");
					String value = tank.getCargoid();
					em.appendChild(document.createTextNode(value));
					subElement.appendChild(em);
				}
			}
			//Misc Tanks
			mainElement = document.createElement("misctanks");
			rootElement.appendChild(mainElement);
			for (Tank tank: ShipData.getInstance().getMiscTankList()){
				Element subElement = document.createElement("item");
				mainElement.appendChild(subElement);
				Element em = document.createElement("volume");
				float fvalue = tank.getVolume();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("density");
				fvalue = tank.getDensity();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("maxpercentage");
				fvalue = tank.getMaxfactor();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
			}
			//Ballast Tanks
			mainElement = document.createElement("ballasttanks");
			rootElement.appendChild(mainElement);
			for (Tank tank: ShipData.getInstance().getBallastList()){
				Element subElement = document.createElement("item");
				mainElement.appendChild(subElement);
				Element em = document.createElement("volume");
				float fvalue = tank.getVolume();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("density");
				fvalue = tank.getDensity();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
				em = document.createElement("maxpercentage");
				fvalue = tank.getMaxfactor();
				em.appendChild(document.createTextNode(String.valueOf(fvalue)));
				subElement.appendChild(em);
			}
			
			//Cargo-cargo comp
			mainElement = document.createElement("cargocargocomp");
			rootElement.appendChild(mainElement);
			CargoComp cargocomp = ShipData.getInstance().getCargoCompList();
			for (int i = 0; i < ShipData.getInstance().getCargoList().size();i++){
				String comparray = "";
				for (int i2 = 0; i2 < ShipData.getInstance().getCargoList().size();i2++){
					if(cargocomp.get(i, i2)){
						comparray += "1";
					} else {
						comparray += "0";
					}
				}
				Element subElement = document.createElement("comparray");
				subElement.appendChild(document.createTextNode(comparray));
				mainElement.appendChild(subElement);
			}
			//Cargo-tank comp
			mainElement = document.createElement("cargotankcomp");
			rootElement.appendChild(mainElement);
			TankComp tankcomp = ShipData.getInstance().getTankCompList();
			for (int i = 0; i < ShipData.getInstance().getCargoList().size();i++){
				String comparray = "";
				for (int i2 = 0; i2 < ShipData.getInstance().getCargoTankList().size();i2++){
					if(tankcomp.get(i, i2)){
						comparray += "1";
					} else {
						comparray += "0";
					}
				}
				Element subElement = document.createElement("comparray");
				subElement.appendChild(document.createTextNode(comparray));
				mainElement.appendChild(subElement);
			}
			StreamResult result = new StreamResult(new StringWriter());
			DOMSource source = new DOMSource(document);
			Transformer xformer = TransformerFactory.newInstance().newTransformer();
			xformer.transform(source, result);

			xmlString = result.getWriter().toString();
			
			} catch( Exception e){
				Log.getInstance().add(Log.JAVA,e);
				JOptionPane.showMessageDialog(null, 
						languageResource.getString("exceptionMessage"),
						languageResource.getString("exceptionTitle"),
						JOptionPane.WARNING_MESSAGE);
			}
			return xmlString;
	}
	
	// Listen for event that are undoable
	protected class MyUndoableEditListener implements UndoableEditListener {
		public void undoableEditHappened(UndoableEditEvent e) {
			undo.addEdit(e.getEdit());
		}
	}
	
	/**
	 * Replace current Doc with a String.
	 * @param s String.
	 */
	public void replace(String s){
		try {
			replace(0, getLength(), s, null);
		} catch (BadLocationException e) {
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(MainFrame.getInstance(), 
				Settings.getInstance().getResourceBundle().getString("exceptionMessage"),
				Settings.getInstance().getResourceBundle().getString("exceptionTitle"),
				JOptionPane.WARNING_MESSAGE);
		}
	}

	/**
	 * Get the current loaded file.
	 * @return File object.
	 */
	public File getCurrentFile() {
		return currentFile;
	}

	/**
	 * Show the current loaded Doc.
	 *
	 */
	public void show(){
		try{
			InputSource in = new InputSource();
			in.setCharacterStream(new StringReader(this.getText(0, getLength())));
			SAXParserFactory factory = SAXParserFactory.newInstance();
			factory.setValidating(true);
			factory.setNamespaceAware(false);
		
			SAXParser parser = factory.newSAXParser();
			parser.parse(in, new FileHandler());
		} catch (Exception ex){
			ex.printStackTrace();
			Log.getInstance().add(Log.JAVA,ex);
			JOptionPane.showMessageDialog(MainFrame.getInstance(), 
					languageResource.getString("parsingException"),
					languageResource.getString("exceptionTitle"),
					JOptionPane.WARNING_MESSAGE);
		}
	}
	
	//	 Class to make loading of files possible
	private class FileHandler extends DefaultHandler{
		private CargoList newCargoList;
		private CargoComp newCargoComp;
		private TankComp newTankComp;
		private final static int CARGO = 0;
		private final static int CARGOTANKS = 1;
		private final static int MISCTANKS = 2;
		private final static int BALLASTTANKS = 3;
		private final static int CARGOCARGOCOMP = 4;
		private final static int CARGOTANKCOMP = 5;
		private int node;
		private int cargotankno;
		private int misctankno;
		private int ballasttankno;
		private int cargocargocompno;
		private int cargotankcompno;
		private boolean floatvalue;
		private boolean stringvalue;
		private boolean hascargoid;
		private float currentFloat;
		private String currentString;
		
		private String description;
		private String cargoid;
		private String comparray;
		private float volume;
		private float density;
		private float weight;
		private float maxfactor;
		private StringBuffer buf;
		
		public FileHandler(){
			newCargoList = ShipData.getInstance().getCargoList();
			newCargoComp = ShipData.getInstance().getCargoCompList();
			newTankComp = ShipData.getInstance().getTankCompList();
			newCargoList.clear();
			newCargoComp.clear();
			newTankComp.clear();
			ShipData.getInstance().getCargoTankList().emptyTanks(true);
			ShipData.getInstance().getMiscTankList().emptyTanks(true);
			ShipData.getInstance().getBallastList().emptyTanks(false);
			floatvalue = false;
			stringvalue = false;
			cargotankno = 0;
			misctankno = 0;
			ballasttankno = 0;
			cargocargocompno = 0;
			cargotankcompno = 0;
			hascargoid = false;
			node = 0;
		}

		public void startElement(String uri, String name, String qname,
				Attributes att) throws SAXException {
			if(qname.equals("cargo")){
				node = CARGO;
			}
			if(qname.equals("cargotanks")){
				node = CARGOTANKS;
			}
			if(qname.equals("misctanks")){
				node = MISCTANKS;
			}
			if(qname.equals("ballasttanks")){
				node = BALLASTTANKS;
			}
			if(qname.equals("cargocargocomp")){
				node = CARGOCARGOCOMP;
			}
			if(qname.equals("cargotankcomp")){
				node = CARGOTANKCOMP;
			}
			if((qname.equals("volume")) || (qname.equals("density"))
					|| (qname.equals("weight")) || (qname.equals("maxpercentage"))){
				floatvalue = true;
			}
			if((qname.equals("description")) || (qname.equals("cargoid"))
					|| (qname.equals("comparray"))){
				stringvalue = true;
			}
			buf = new StringBuffer();
		}
		
		public void characters(char[] ch, int start, int length){
			if(buf != null){
				buf.append(ch, start, length);
			}	
		}
		
		public void endElement(String uri, String name, String qname)
		throws SAXException {
			if(floatvalue == true){
				currentFloat = Float.valueOf(buf.toString());
				floatvalue = false;
			}
			if(stringvalue == true){
				currentString = buf.toString();
				stringvalue = false;
			}
			if(qname.equals("volume")){
				volume = currentFloat;
			}
			if(qname.equals("weight")){
				weight = currentFloat;
			}
			if(qname.equals("density")){
				density = currentFloat;
			}
			if(qname.equals("maxpercentage")){
				maxfactor = currentFloat;
			}
			if(qname.equals("cargoid")){
				cargoid = currentString;
				hascargoid = true;
			}
			if(qname.equals("description")){
				description = currentString;
			}
			if(qname.equals("comparray")){
				comparray = currentString;
			}
			if((qname.equals("item")) && (node == CARGO)){
				newCargoList.add(
						new Cargo(cargoid, description, weight, density),
						MainFrame.getInstance());
				newCargoComp.addComp();
				newTankComp.addComp();
				hascargoid = false;
			}
			if((qname.equals("item")) && (node == CARGOTANKS)){
				Tank cargotank = ShipData.getInstance().
					getCargoTankList().get(cargotankno);
				if(hascargoid){
					cargotank.setCargoid(cargoid);
					hascargoid = false;
				}
				cargotank.setVolume(volume, density);
				cargotank.setMaxfactor(maxfactor);
				cargotankno++;
			}
			if((qname.equals("item")) && (node == MISCTANKS)){
				Tank misctank = ShipData.getInstance().
					getMiscTankList().get(misctankno);
				misctank.setVolume(volume, density);
				misctank.setMaxfactor(maxfactor);
				misctankno++;
			}
			if((qname.equals("item")) && (node == BALLASTTANKS)){
				Tank ballasttank = ShipData.getInstance()
					.getBallastList().get(ballasttankno);
				ballasttank.setVolume(volume, density);
				ballasttank.setMaxfactor(maxfactor);
				ballasttankno++;
			}
			if((qname.equals("comparray")) && (node == CARGOCARGOCOMP)){
				for(int i = 0; i < newCargoList.size(); i++){
					boolean value = true;
					if(comparray.charAt(i) == '0'){
						value = false;
					}
					newCargoComp.set(value, cargocargocompno, i);
				}
				cargocargocompno++;
			}
			if((qname.equals("comparray")) && (node == CARGOTANKCOMP)){
				for(int i = 0; i < ShipData.getInstance().
					getCargoTankList().size(); i++){
					boolean value = true;
					if(comparray.charAt(i) == '0'){
						value = false;
					}
					newTankComp.set(value, cargotankcompno, i);
				}
				cargotankcompno++;
			}
		}
			
		public void endDocument() throws SAXException{
			MainFrame.getInstance().updateInterface();
		}
	}

	/**
	 * See if condition changed.
	 * @return True if changed, otherwise false.
	 */
	public boolean isCanged(){
		return changed;
	}

	/**
	 * See if an action can be undone.
	 * @return True if undone can be undone, otherwise false.
	 */
	public boolean canUndo(){
		return undo.canUndo();
	}

	/**
	 * See if an action can be redone.
	 * @return True if undone can be redone, otherwise false.
	 */
	public boolean canRedo(){
		return undo.canRedo();
	}

	/**
	 * See if save has to be enabled.
	 * @return True if it has to be enabled, otherwise false.
	 */
	public boolean isSave(){
		if((currentFile != null) && (changed == true)){
			return true;
		} else {
			return false;
		}
	}

	/**
	 * Save Doc to file.
	 */
	public void save(){
		writeFile(currentFile);
		changed = false;
		MainFrame.getInstance().updateMenu();
	}

	/**
	 * Save Doc to file as
	 * @param file File you want it to save under.
	 */
	public void saveAs(File file){
		currentFile = file;
		writeFile(file);
		changed = false;
		MainFrame.getInstance().setFileName(file);
		MainFrame.getInstance().updateMenu();
	}
	
	// Write Doc to file
	private void writeFile(File file){
		try {
			FileWriter fwriter = new FileWriter(file,false);
			fwriter.write(getText(0, getLength()));
			fwriter.close();
		} catch (Exception e) {
			Log.getInstance().add(Log.JAVA,e);
			JOptionPane.showMessageDialog(MainFrame.getInstance(), 
				Settings.getInstance().getResourceBundle().getString("exceptionMessage"),
				Settings.getInstance().getResourceBundle().getString("exceptionTitle"),
				JOptionPane.WARNING_MESSAGE);
		}
	}

	/**
	 * Update to current shown condition.
	 */
	public void update(){
		String currentShown = getCurrent();
		String current = null;
		try {
			current = getText(0, getLength());
		} catch (BadLocationException e) {
			Log.getInstance().add(Log.JAVA, e);
			e.printStackTrace();
		}
		if(!current.equals(currentShown)){
			changed = true;
			replace(currentShown);
		}
	}

	/**
	 * Undo previously taken action.
	 */
	public void undo(){
		undo.undo();
		undo.undo();
		show();
		changed = true;
		MainFrame.getInstance().updateMenu();
	}

	/**
	 * Redo previously taken action.
	 */
	public void redo(){
		undo.redo();
		undo.redo();
		show();
		changed = true;
		MainFrame.getInstance().updateMenu();
	}
}
