//: be/optiloading/helpers/table/NumberEditor.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.helpers.table;

import java.awt.Component;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.text.NumberFormat;
import java.text.ParseException;

import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import be.optiloading.gui.Log;
import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;

/**
 * NumberEditor to edit numbers in table cells.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class NumberEditor extends DefaultCellEditor {
    JFormattedTextField ftf;
    NumberFormat formatter;

    /**
     * Constructor.
     */
    public NumberEditor(int fractionDigits) {
        super(new JFormattedTextField());
        ftf = (JFormattedTextField)getComponent();

        //Set up the editor for the integer cells.
        formatter = NumberFormat.getInstance();
        formatter.setMinimumFractionDigits(fractionDigits);
        NumberFormatter factory = new NumberFormatter(formatter);
        factory.setMinimum(0f);

        ftf.setFormatterFactory(
                new DefaultFormatterFactory(factory));
        ftf.setHorizontalAlignment(JTextField.TRAILING);
        ftf.setFocusLostBehavior(JFormattedTextField.PERSIST);

        ftf.getInputMap().put(KeyStroke.getKeyStroke(
                                        KeyEvent.VK_ENTER, 0),
                                        "check");
        ftf.getActionMap().put("check", new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
            	try { 
                    ftf.commitEdit();
                    ftf.postActionEvent();
                } catch (java.text.ParseException exc) { 
    				Log.getInstance().add(Log.JAVA,exc);
                }
                stopCellEditing();
            }
        });
    }

    /**
     * Override to invoke setValue on the formatted text field.
     */
    public Component getTableCellEditorComponent(JTable table,
            Object value, boolean isSelected,
            int row, int column) {
        JFormattedTextField ftf =
            (JFormattedTextField)super.getTableCellEditorComponent(
                table, value, isSelected, row, column);
        ftf.setValue(value);
        return ftf;
    }

    /**
     * Override to ensure that the value remains a Float.
     */
    public Object getCellEditorValue() {
        JFormattedTextField ftf = (JFormattedTextField)getComponent();
        Object o = ftf.getValue();
        if (o instanceof Float) {
            return o;
        } else if (o instanceof Number) {
            return new Float(((Number)o).floatValue());
        } else {
            try {
                return formatter.parseObject(o.toString());
            } catch (ParseException exc) {
            	JOptionPane.showMessageDialog(MainFrame.getInstance(),
            			Settings.getInstance().
            			getResourceBundle().getString("invalidValue"));
				Log.getInstance().add(Log.JAVA,exc);
            	return null;
            }
        }
    }

    /**
     * Called when user leaves cell.
     */
    public boolean stopCellEditing() {
        JFormattedTextField ftf = (JFormattedTextField)getComponent();
        if (ftf.isEditValid()) {
            try {
                ftf.commitEdit();
            } catch (java.text.ParseException exc) {
            	JOptionPane.showMessageDialog(MainFrame.getInstance(),
            			Settings.getInstance().
            			getResourceBundle().getString("invalidValue"));
				Log.getInstance().add(Log.JAVA,exc);
            }
        } else {
        	JOptionPane.showMessageDialog(MainFrame.getInstance(),
        			Settings.getInstance().
        			getResourceBundle().getString("invalidValue"));
        }
        return super.stopCellEditing();
    }
}
