//: be/optiloading/helpers/table/PercentageSpinnerEd.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.helpers.table;

import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseEvent;
import java.text.DecimalFormat;
import java.text.ParseException;
import java.util.EventObject;

import javax.swing.AbstractCellEditor;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JOptionPane;
import javax.swing.JSpinner;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.table.TableCellEditor;
import javax.swing.text.DefaultFormatterFactory;
import javax.swing.text.NumberFormatter;

import be.optiloading.gui.MainFrame;
import be.optiloading.settings.Settings;
import be.optiloading.tank.TankModel;

/**
 * PercentageSpinnerEditor to edit numbers in table cells.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class PercentageSpinnerEditor extends AbstractCellEditor implements TableCellEditor {
	private JSpinner spinner;
	private OptionalPercentSpinEditor editor;

	/**
	 * Initializes the spinner.
	 */
	public PercentageSpinnerEditor() {
		spinner = new JSpinner(new SpinnerNumberModel(0, 0, 1, .01));
		editor = new OptionalPercentSpinEditor(spinner);
		spinner.setEditor(editor);
		((OptionalPercentSpinEditor)spinner.getEditor()).getTextField()
			.setForeground(new Color(9,42,212));
		spinner.setBorder(null);
		spinner.getEditor().getParent().setFont(new Font("Dialog",Font.PLAIN,12));
		((OptionalPercentSpinEditor)spinner.getEditor()).getTextField().
			addKeyListener(new KeyAdapter(){
				public void keyPressed(KeyEvent e) {
					if(e.getKeyCode() == KeyEvent.VK_ENTER){
						try {
						    spinner.commitEdit();
						} catch (ParseException pe) {
						    JComponent editor = spinner.getEditor();
						     if (editor instanceof OptionalPercentSpinEditor) {
						        ((OptionalPercentSpinEditor)editor).getTextField()
						        	.setValue(spinner.getValue());
						    }
						     JOptionPane.showMessageDialog(MainFrame.getInstance(),
				            			Settings.getInstance().
				            			getResourceBundle().getString("invalidValue"));
						}
					}
				}			
		});
	}

	/**
     * Prepares the spinner component and returns it.
     */ 
	public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int column) {
		boolean cargo = ((TankModel) table.getModel()).isCargo();
		if(cargo && (column != 7)){
			float maxvalue = (Float) ((TankModel) table.getModel())
				.getValueAt(row, 7);
			spinner.setModel(new SpinnerNumberModel(0, 0, maxvalue, .01));
		} else if((!cargo) && (column != 6)) {
			float maxvalue = (Float) ((TankModel) table.getModel())
				.getValueAt(row, 6);
			spinner.setModel(new SpinnerNumberModel(0, 0, maxvalue, .01));
		}
		spinner.setValue(value);
		return spinner;
	}

	/**
	 * Returns the spinners current value.
	 */ 
	public Object getCellEditorValue() {
		try {
		    spinner.commitEdit();
		} catch (ParseException pe) {
		    JComponent editor = spinner.getEditor();
		     if (editor instanceof OptionalPercentSpinEditor) {
		        ((OptionalPercentSpinEditor)editor).getTextField().setValue(spinner.getValue());
		    }
		     JOptionPane.showMessageDialog(MainFrame.getInstance(),
         			Settings.getInstance().
         			getResourceBundle().getString("invalidValue"));
		}
		return spinner.getValue();
	}
	
	/**
	 * Enables the editor only for double-clicks.
	 */
	public boolean isCellEditable(EventObject evt) {
		if (evt instanceof MouseEvent) {
			return ((MouseEvent) evt).getClickCount() >= 2;
		}
		return true;
	}

	// -------- begin static nested classes --------

	private static class OptionalPercentFormat extends DecimalFormat {
		public OptionalPercentFormat() {
			super("0.00%");
			setMultiplier(100);
		}

		public Object parseObject(String source) throws ParseException {
			if (!source.endsWith("%"))
				source += "%";
			return super.parseObject(source);
		}
	}

	private static class OptionalPercentSpinEditor extends
			JSpinner.DefaultEditor {
		public OptionalPercentSpinEditor(JSpinner spinner) {
			super(spinner);
			NumberFormatter formatter = new NumberFormatter(
					new OptionalPercentFormat());
			formatter.setValueClass(Double.class);
			JFormattedTextField ftf = getTextField();
			ftf.setFormatterFactory(new DefaultFormatterFactory(formatter));
			ftf.setEditable(true);
			ftf.setHorizontalAlignment(JTextField.RIGHT);
		}
	}
}
