//: /be/optiloading/optimization/Section.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.optimization;

/**
 * Section to get the load between two points.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

public class Section {
	private float fwdbound;
	private float aftbound;
	private float[] tankfactors;
	private float buoymisc;
	private float maxforce;
	
	/**
	 * Constructor
	 * @param fwdbound Floating point indicating fwd bound.
	 * @param aftbound Floating point indicating aft bound.
	 * @param tankfactors Float array indicating which tanks 
	 * 			are located in this section.
	 * @param buoymisc Foating point containing corrected buoyancy.
	 * @param maxforce Maximum force possible on this compartment.
	 */
	public Section(float fwdbound, float aftbound, float[] tankfactors,
			float buoymisc, float maxforce){
		this.fwdbound = fwdbound;
		this.aftbound = aftbound;
		this.tankfactors = tankfactors;
		this.buoymisc = buoymisc;
		this.maxforce = maxforce;
	}
	
	/**
	 * Get fwd bound.
	 * @return Floating point value with fwd bound.
	 */
	public float getFwdbound() {
		return fwdbound;
	}
	
	/**
	 * Get aft bound.
	 * @return Floating point value with aft bound.
	 */
	public float getAftbound() {
		return aftbound;
	}
	
	/**
	 * Tankfactors indicating which tanks are in this section.
	 * @return Float array with tankfactors.
	 */
	public float getTankfactors(int tank) {
		return tankfactors[tank];
	}

	/**
	 * Get corrected buoyancy.
	 * @return Foating point containing corrected buoyancy.
	 */
	public float getBuoymisc() {
		return buoymisc;
	}
	
	/**
	 * Get maximum force on this section.
	 * @return Floating point value containing maximum force.
	 */
	public float getMaxforce() {
		return maxforce;
	}
}