//: /be/optiloading/settings/GeneralPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.settings;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Toolkit;
import java.text.NumberFormat;
import java.text.ParseException;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.InputVerifier;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * Panel for general settings.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
class GeneralPanel extends JPanel{

	private NumberFormat formatter;

	JFormattedTextField inputtrim;
	JFormattedTextField inputheel;
	JFormattedTextField waterdensfield;
	JFormattedTextField ballastwaterdensfield;
	
	JComboBox languagebox;
	
	private ResourceBundle languageResource;
	
	/**
	 * Constructor to build panel.
	 */
	public GeneralPanel(){
		languageResource = Settings.languageResource;
		
		Dimension shortField = new Dimension( 80, 22 );
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		
		// Input Panel
		JPanel input = new JPanel();
		input.setLayout(new GridBagLayout());
		

		//Desired trim
		JLabel trimlabel = new JLabel(languageResource.getString("desiredTrimLabel")+":");
		input.add(trimlabel,c);
		formatter = NumberFormat.getInstance();
		formatter.setMinimumFractionDigits(2);
		inputtrim = new JFormattedTextField(formatter);
		
		inputtrim.setInputVerifier(new InputVerifier() {
			public boolean verify(JComponent input) {
				return checkField(input, false);
			}

			protected void makeItPretty(JComponent input) {
				checkField(input, true);
			}

			public boolean checkField(JComponent comp, boolean change) {
				boolean wasValid = true;
				float desiredtrim = ((Number) inputtrim.getValue()).floatValue();

				try {
					desiredtrim = (float) formatter.parse(inputtrim.getText())
							.doubleValue();
				} catch (ParseException pe) {
					wasValid = false;
				}

				if ((desiredtrim < -5) || (desiredtrim > 5)) {
					wasValid = false;
					if (change) {
						if (desiredtrim < -5) {
							desiredtrim = -5;
						} else {
							desiredtrim = 5;
						}
					}
				}

				if (change) {
					inputtrim.setText(formatter.format(desiredtrim));
					inputtrim.selectAll();
				}

				return wasValid;
			}

			public boolean shouldYieldFocus(JComponent input) {
				boolean inputOK = verify(input);
				makeItPretty(input);

				if (inputOK) {
					return true;
				} else {
					Toolkit.getDefaultToolkit().beep();
					return false;
				}
			}

		});
		inputtrim.setHorizontalAlignment(JTextField.RIGHT);
		inputtrim.setToolTipText(languageResource.getString("desiredTrimTooltip"));
		inputtrim.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(inputtrim, c);
		
	    
	    //Desired heel
		JLabel heellabel = new JLabel(languageResource.getString("desiredHeelLabel")+":");
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
		input.add(heellabel,c);
		inputheel = new JFormattedTextField(formatter);
		inputheel.setInputVerifier(new InputVerifier() {
			public boolean verify(JComponent input) {
				return checkField(input, false);
			}

			protected void makeItPretty(JComponent input) {
				checkField(input, true);
			}

			public boolean checkField(JComponent comp, boolean change) {
				boolean wasValid = true;
				float desiredheel = ((Number) inputheel.getValue())
						.floatValue();

				try {
					desiredheel = (float) formatter.parse(inputheel.getText())
							.doubleValue();
				} catch (ParseException pe) {
					wasValid = false;
				}

				if ((desiredheel < -25) || (desiredheel > 25)) {
					wasValid = false;
					if (change) {
						if (desiredheel < -25) {
							desiredheel = -25;
						} else {
							desiredheel = 25;
						}
					}
				}

				if (change) {
					inputheel.setText(formatter.format(desiredheel));
					inputheel.selectAll();
				}

				return wasValid;
			}

			public boolean shouldYieldFocus(JComponent input) {
				boolean inputOK = verify(input);
				makeItPretty(input);

				if (inputOK) {
					return true;
				} else {
					Toolkit.getDefaultToolkit().beep();
					return false;
				}
			}

		});
		inputheel.setHorizontalAlignment(JTextField.RIGHT);
		inputheel.setToolTipText(languageResource.getString("desiredHeelTooltip"));
		inputheel.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(inputheel, c);
		
	    //Water density
		JLabel waterdensity = new JLabel(languageResource.getString("waterDensityLabel"));
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
		input.add(waterdensity,c);
		SpinnerNumberModel specgravmod = new SpinnerNumberModel(1.025,1.000-0.010,1.000+0.035,0.001);
		JSpinner waterdens = new JSpinner(specgravmod);
		waterdens.setToolTipText(languageResource.getString("waterDensityTooltip"));
		waterdens.setEditor(new JSpinner.NumberEditor(waterdens, "0.000"));
		waterdensfield = ((JSpinner.DefaultEditor) waterdens.getEditor()).getTextField();
		waterdensfield.setFont(new Font("Dialog",Font.PLAIN,12));
		waterdens.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(waterdens, c);
		
		//Ballast water density
		JLabel ballastwaterdensity = new JLabel(languageResource.getString("ballastDensityLabel"));
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
		input.add(ballastwaterdensity,c);
		SpinnerNumberModel specgravmod2 = new SpinnerNumberModel(1.025,1.000-0.010,1.000+0.035,0.001);
		JSpinner ballastwaterdens = new JSpinner(specgravmod2);
		ballastwaterdens.setToolTipText(languageResource.getString("ballastDensityTooltip"));
		ballastwaterdens.setEditor(new JSpinner.NumberEditor(ballastwaterdens, "0.000"));
		ballastwaterdensfield = ((JSpinner.DefaultEditor) ballastwaterdens.getEditor()).getTextField();
		ballastwaterdensfield.setFont(new Font("Dialog",Font.PLAIN,12));
		ballastwaterdens.setPreferredSize(shortField);
		c.gridx = 1;
	    c.gridwidth = 3; 
	    input.add(ballastwaterdens, c);
		
		//Container panel for input panel
		JPanel inputmain = new JPanel(new BorderLayout());
		inputmain.add(BorderLayout.CENTER,input);
		inputmain.add(BorderLayout.SOUTH, Box.createVerticalStrut(10));
		inputmain.setBorder(new CompoundBorder(
				new TitledBorder(new EtchedBorder(),languageResource.getString("inputTitle")),
				new EmptyBorder(5,5,0,5)));

		// Language panel
		JPanel languagepanel = new JPanel();
		languagepanel.setLayout(new BoxLayout(languagepanel, BoxLayout.X_AXIS));
		languagepanel.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("languageTitle")), 
				new EmptyBorder(1, 5, 3, 5)));

		String[] supportedLanguages = { "English", "Dutch (Nederlands)"};
		languagebox = new JComboBox(supportedLanguages);
		languagebox.setToolTipText(languageResource.getString("languageTooltip"));
		languagebox.setAlignmentX(SwingConstants.LEFT);
		languagepanel.add(languagebox);
		
		JPanel main = new JPanel();
		main.setLayout(new BorderLayout());
		main.add(BorderLayout.CENTER,inputmain);
		main.add(BorderLayout.SOUTH,languagepanel);
		
		setLayout(new BorderLayout());
		add(BorderLayout.NORTH,main);
	}
}
