//: /be/optiloading/settings/SaPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2008 Tim Pintens. All rights reserved.
 */

package be.optiloading.settings;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ResourceBundle;

import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JSpinner;
import javax.swing.SpinnerNumberModel;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;

/**
 * Panel for simulated annealing settings.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class SaPanel extends JPanel{
	//Objective
	JCheckBox usedtanks;
	JCheckBox strength;
	JCheckBox trim;
	JCheckBox heel;
	JCheckBox compatibility;
	
	JFormattedTextField heelfield;
	JFormattedTextField compatibilityfield;
	JFormattedTextField trimfield;
	JFormattedTextField usedtanksfield;
	JFormattedTextField strengthfield;
	
	//SA Settings
	JFormattedTextField coolingratefield;
	JFormattedTextField trailspertempfield;
	JFormattedTextField goodswapspertempfield;
	JFormattedTextField maxiterationsfield;
	JFormattedTextField starttempfactorfield;
	JFormattedTextField numberofrestartsfield;
	
	private ResourceBundle languageResource;
	private JSpinner trimspinner;
	private JSpinner heelspinner;
	private JSpinner compatibilityspinner;
	private JSpinner usedtanksspinner;
	private JSpinner strengthspinner;
	private JSpinner coolingratespinner;
	private JSpinner trailspertempspinner;
	private JSpinner goodswapspertempspinner;
	private JSpinner maxiterationsspinner;
	private JSpinner starttempfactorspinner;
	private JSpinner numberofrestartsspinner;
	
	/**
	 * Constructor to build panel.
	 */
	public SaPanel(){
		languageResource = Settings.languageResource;
		
		//	Criteria Panel
		JPanel criteria = new JPanel();
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		criteria.setLayout(new GridBagLayout());
		criteria.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("criteriaTitle")),
				new EmptyBorder(1, 5, 3, 5)));
		
		// Used tanks
		usedtanks = new JCheckBox(languageResource.getString("criteriaUsedtanks"), null, true);
		usedtanks.addItemListener(new CriteriaListener());
		usedtanks.setToolTipText(languageResource.
				getString("criteriaUsedtanksTooltip"));
		usedtanks.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(usedtanks.isSelected()){
					usedtanksfield.setEnabled(true);
					usedtanksspinner.setEnabled(true);
				} else {
					usedtanksfield.setEnabled(false);
					usedtanksspinner.setEnabled(false);
				}
			}
		});
		criteria.add(usedtanks,c);
		SpinnerNumberModel usedtanksmod = new SpinnerNumberModel(1,1,100,1);
		usedtanksspinner = new JSpinner(usedtanksmod);
		usedtanksspinner.setEditor(new JSpinner.NumberEditor(usedtanksspinner, "0"));
		usedtanksspinner.setToolTipText(languageResource.getString("usedtankscriTooltip"));
		usedtanksfield = ((JSpinner.DefaultEditor) usedtanksspinner.getEditor()).getTextField();
		usedtanksfield.setFont(new Font("Dialog",Font.PLAIN,12));
		usedtanksspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		criteria.add(usedtanksspinner, c);
		
		// Strength
		strength = new JCheckBox(languageResource.getString("criteriaStress"), null, true);
		strength.addItemListener(new CriteriaListener());
		strength.setToolTipText(languageResource.
				getString("criteriaStresTooltip"));
		strength.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(strength.isSelected()){
					strengthfield.setEnabled(true);
					strengthspinner.setEnabled(true);
				} else {
					strengthfield.setEnabled(false);
					strengthspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
	    criteria.add(strength,c);
		SpinnerNumberModel strengthmod = new SpinnerNumberModel(1,1,100,1);
		strengthspinner = new JSpinner(strengthmod);
		strengthspinner.setEditor(new JSpinner.NumberEditor(strengthspinner, "0"));
		strengthspinner.setToolTipText(languageResource.getString("strengthcriTooltip"));
		strengthfield = ((JSpinner.DefaultEditor) strengthspinner.getEditor()).getTextField();
		strengthfield.setFont(new Font("Dialog",Font.PLAIN,12));
		strengthspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
		criteria.add(strengthspinner, c);
		
		// Trim
		trim = new JCheckBox(languageResource.getString("criteriaTrim"), null, true);
		trim.addItemListener(new CriteriaListener());
		trim.setToolTipText(languageResource.
				getString("criteriaTrimTooltip"));
		trim.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(trim.isSelected()){
					trimfield.setEnabled(true);
					trimspinner.setEnabled(true);
				} else {
					trimfield.setEnabled(false);
					trimspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
	    criteria.add(trim,c);
		SpinnerNumberModel trimmod = new SpinnerNumberModel(1,1,100,1);
		trimspinner = new JSpinner(trimmod);
		trimspinner.setEditor(new JSpinner.NumberEditor(trimspinner, "0"));
		trimspinner.setToolTipText(languageResource.getString("trimcriTooltip"));
		trimfield = ((JSpinner.DefaultEditor) trimspinner.getEditor()).getTextField();
		trimfield.setFont(new Font("Dialog",Font.PLAIN,12));
		trimspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
		criteria.add(trimspinner, c);
	    
		// Heel
		heel = new JCheckBox(languageResource.getString("criteriaHeel"), null, true);
		heel.addItemListener(new CriteriaListener());
		heel.setToolTipText(languageResource.
				getString("criteriaHeelTooltip"));
		heel.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(heel.isSelected()){
					heelfield.setEnabled(true);
					heelspinner.setEnabled(true);
				} else {
					heelfield.setEnabled(false);
					heelspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
	    criteria.add(heel,c);
		SpinnerNumberModel heelmod = new SpinnerNumberModel(1,1,100,1);
		heelspinner = new JSpinner(heelmod);
		heelspinner.setEditor(new JSpinner.NumberEditor(heelspinner, "0"));
		heelspinner.setToolTipText(languageResource.getString("heelcriTooltip"));
		heelfield = ((JSpinner.DefaultEditor) heelspinner.getEditor()).getTextField();
		heelfield.setFont(new Font("Dialog",Font.PLAIN,12));
		heelspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
		criteria.add(heelspinner, c);
		
		// Compatibility
		compatibility = new JCheckBox(languageResource.getString("criteriaCompatibility"), null, true);
		compatibility.addItemListener(new CriteriaListener());
		compatibility.setToolTipText(languageResource.
				getString("criteriaCompatibilityTooltip"));
		compatibility.addItemListener(new ItemListener(){
			public void itemStateChanged(ItemEvent e) {
				if(compatibility.isSelected()){
					compatibilityfield.setEnabled(true);
					compatibilityspinner.setEnabled(true);
				} else {
					compatibilityfield.setEnabled(false);
					compatibilityspinner.setEnabled(false);
				}
			}
		});
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 4;
	    criteria.add(compatibility,c);
		SpinnerNumberModel compatibilitymod = new SpinnerNumberModel(1,1,100,1);
		compatibilityspinner = new JSpinner(compatibilitymod);
		compatibilityspinner.setEditor(new JSpinner.NumberEditor(compatibilityspinner, "0"));
		compatibilityspinner.setToolTipText(languageResource.getString("compatibilitycriTooltip"));
		compatibilityfield = ((JSpinner.DefaultEditor) compatibilityspinner.getEditor()).getTextField();
		compatibilityfield.setFont(new Font("Dialog",Font.PLAIN,12));
		compatibilityspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
	    c.gridwidth = 3;
		criteria.add(compatibilityspinner, c);
		
		//	SA settings Panel
		JPanel sasettings = new JPanel();
		c = new GridBagConstraints();
		c.insets = new Insets( 2, 2, 2, 2 );
		c.anchor = GridBagConstraints.WEST;
		sasettings.setLayout(new GridBagLayout());
		sasettings.setBorder(new CompoundBorder(new TitledBorder(
				new EtchedBorder(), languageResource.getString("sasettingsTitle")),
				new EmptyBorder(1, 5, 3, 5)));
		
		// starttempfactor
		sasettings.add(new JLabel(languageResource.getString("starttempfactorLabel")),c);
		SpinnerNumberModel starttempfactormod = new SpinnerNumberModel(100,1,1000,10);
		starttempfactorspinner = new JSpinner(starttempfactormod);
		starttempfactorspinner.setEditor(new JSpinner.NumberEditor(starttempfactorspinner, "0"));
		starttempfactorspinner.setToolTipText(languageResource.getString("starttempfactorTooltip"));
		starttempfactorfield = ((JSpinner.DefaultEditor) starttempfactorspinner.getEditor()).getTextField();
		starttempfactorfield.setFont(new Font("Dialog",Font.PLAIN,12));
		starttempfactorspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(starttempfactorspinner, c);
		
		
		// coolingrate
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 1;
	    sasettings.add(new JLabel(languageResource.getString("coolingrateLabel")),c);
		SpinnerNumberModel coolingratemod = new SpinnerNumberModel(0.95,0.01,0.99,0.01);
		coolingratespinner = new JSpinner(coolingratemod);
		coolingratespinner.setEditor(new JSpinner.NumberEditor(coolingratespinner, "0.00"));
		coolingratespinner.setToolTipText(languageResource.getString("coolingrateTooltip"));
		coolingratefield = ((JSpinner.DefaultEditor) coolingratespinner.getEditor()).getTextField();
		coolingratefield.setFont(new Font("Dialog",Font.PLAIN,12));
		coolingratespinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(coolingratespinner, c);
	    
		
		// trailspertemp
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 2;
		sasettings.add(new JLabel(languageResource.getString("trailspertempLabel")),c);
		SpinnerNumberModel trailspertempmod = new SpinnerNumberModel(300,1,10000,10);
		trailspertempspinner = new JSpinner(trailspertempmod);
		trailspertempspinner.setEditor(new JSpinner.NumberEditor(trailspertempspinner, "0"));
		trailspertempspinner.setToolTipText(languageResource.getString("trailspertempTooltip"));
		trailspertempfield = ((JSpinner.DefaultEditor) trailspertempspinner.getEditor()).getTextField();
		trailspertempfield.setFont(new Font("Dialog",Font.PLAIN,12));
		trailspertempspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(trailspertempspinner, c);
		
		
		// goodswapspertemp
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 3;
		sasettings.add(new JLabel(languageResource.getString("goodswapspertempLabel")),c);
		SpinnerNumberModel goodswapspertempmod = new SpinnerNumberModel(100,1,1000,5);
		goodswapspertempspinner = new JSpinner(goodswapspertempmod);
		goodswapspertempspinner.setEditor(new JSpinner.NumberEditor(goodswapspertempspinner, "0"));
		goodswapspertempspinner.setToolTipText(languageResource.getString("goodswapspertempTooltip"));
		goodswapspertempfield = ((JSpinner.DefaultEditor) goodswapspertempspinner.getEditor()).getTextField();
		goodswapspertempfield.setFont(new Font("Dialog",Font.PLAIN,12));
		goodswapspertempspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(goodswapspertempspinner, c);
		
		// maxiterations
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 4;
		sasettings.add(new JLabel(languageResource.getString("maxiterationsLabel")),c);
		SpinnerNumberModel maxiterationsmod = new SpinnerNumberModel(1000,1,10000,10);
		maxiterationsspinner = new JSpinner(maxiterationsmod);
		maxiterationsspinner.setEditor(new JSpinner.NumberEditor(maxiterationsspinner, "0"));
		maxiterationsspinner.setToolTipText(languageResource.getString("maxiterationsTooltip"));
		maxiterationsfield = ((JSpinner.DefaultEditor) maxiterationsspinner.getEditor()).getTextField();
		maxiterationsfield.setFont(new Font("Dialog",Font.PLAIN,12));
		maxiterationsspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(maxiterationsspinner, c);
		
		// numberofrestarts
		c.gridwidth = 1;
	    c.gridx = 0;
	    c.gridy = 5;
		sasettings.add(new JLabel(languageResource.getString("numberofrestartsLabel")),c);
		SpinnerNumberModel numberofrestartsmod = new SpinnerNumberModel(10,0,100,1);
		numberofrestartsspinner = new JSpinner(numberofrestartsmod);
		numberofrestartsspinner.setEditor(new JSpinner.NumberEditor(numberofrestartsspinner, "0"));
		numberofrestartsspinner.setToolTipText(languageResource.getString("numberofrestartsTooltip"));
		numberofrestartsfield = ((JSpinner.DefaultEditor) numberofrestartsspinner.getEditor()).getTextField();
		numberofrestartsfield.setFont(new Font("Dialog",Font.PLAIN,12));
		numberofrestartsspinner.setPreferredSize(new Dimension( 80, 22 ));
		c.gridx = 1;
		c.gridwidth = 3;
		sasettings.add(numberofrestartsspinner, c);
		
		JPanel main = new JPanel();
		main.setLayout(new BorderLayout());
		main.add(BorderLayout.WEST, criteria);
		main.add(BorderLayout.CENTER,sasettings);

		
		setLayout(new BorderLayout());
		add(BorderLayout.NORTH,main);
	}
	
	//	 ItemListener that checks that at least one criteria is selected
	class CriteriaListener implements ItemListener {
		public void itemStateChanged(ItemEvent e) {
			JCheckBox source = (JCheckBox) e.getSource();
			if ((strength.isSelected() == false)
					&& (usedtanks.isSelected() == false)
					&& (trim.isSelected() == false)
					&& (heel.isSelected() == false)
					&& (compatibility.isSelected() == false)) {
				source.setSelected(true);
				JOptionPane.showMessageDialog(Settings.getInstance().getDialog(),
						languageResource.getString("messageCriteria"));
			}
		}
	}
}
