//: /be/optiloading/ship/StabilityGraph.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.ship;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.geom.GeneralPath;
import java.text.NumberFormat;
import java.util.ResourceBundle;

import javax.swing.JPanel;

import be.optiloading.settings.Settings;

/**
 * SabilityGraph class used to draw the GZ curve.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class StabilityGraph extends JPanel{
	private float[][] kn;
	
	/**
	 * Paint the GZ curve.
	 */
	public void paintComponent(Graphics g){
		ResourceBundle languageResource = Settings.getInstance().getResourceBundle();
		
		//Ship Data
		ShipData data = ShipData.getInstance();
		float VCG = data.getVcg();
		float TCG =  Math.abs(data.getTcg());
		float KM =  data.getKm();
		float FSC = data.getFsc();
		kn = data.getKn();
		
		//Graph settings
		int width = this.getSize().width;
		int height =  this.getSize().height;
		int distfromside = 40;
		int topmarge = 15;
		
		float maxangle = kn[kn.length-1][0];
		
		float add = 0f;
		if	(((KM-VCG)-new Float(KM-VCG).intValue()) <= 0.5f){
			add=0.5f;
		}else{
			add=1f;
		}
		float maxgz = new Float(KM-VCG).intValue() + add;
		
		float xdiv = (width - (2*distfromside) - 15)/maxangle;
		float ydiv = (height -(distfromside + topmarge) -15)/maxgz;
		
		Graphics2D g2d = (Graphics2D)g;
		RenderingHints rh = new RenderingHints(
		RenderingHints.KEY_INTERPOLATION,
		RenderingHints.VALUE_INTERPOLATION_BICUBIC);
		rh.put(RenderingHints.KEY_ANTIALIASING, RenderingHints.VALUE_ANTIALIAS_ON);
		rh.put(RenderingHints.KEY_RENDERING, RenderingHints.VALUE_RENDER_QUALITY);
		g2d.setRenderingHints(rh);
		
		g2d.setColor(new Color(194,217,240));
		g2d.fillRect(2, 2, width-4, height-4); //Background
		
		g2d.setColor(new Color(12,72,133));
		g2d.setStroke(new BasicStroke(1.5f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		//X-axis
		g2d.drawLine(distfromside, height-distfromside, 
				width-distfromside, height-distfromside);
		g2d.drawLine(width-distfromside-5, height-distfromside-5, 
				width-distfromside, height-distfromside);
		g2d.drawLine(width-distfromside, height-distfromside, 
				width-distfromside-5, height-distfromside+5);
		//Y-Axis
		g2d.drawLine(distfromside, topmarge, distfromside, 
				height-distfromside);
		g2d.drawLine(distfromside-5, topmarge+5, distfromside, topmarge);
		g2d.drawLine(distfromside, topmarge, distfromside+5, topmarge+5);
		
		g2d.setFont(new Font("Dialog",Font.PLAIN,10));
		//X-axis markings
		for(int i = 0;i <= maxangle;i = i +10){
			g2d.drawLine(new Float(distfromside + (i*xdiv)).intValue(), height-distfromside-5, 
					new Float(distfromside + (i*xdiv)).intValue(), height-distfromside+5);
			g2d.drawString(String.valueOf(i), new Float(distfromside - 5 + (i*xdiv)).intValue(), 
					height-distfromside+15);
		}
		g2d.drawString(languageResource.getString("gzAngle"), (width/2)-20, height - distfromside + 30);
		
		//Y-axis markings
		for(float i=0;i <=maxgz;i = i + 0.5f){
			g2d.drawLine(distfromside-5, new Float(height-distfromside-(i*ydiv)).intValue()
					, distfromside+5, new Float(height-distfromside-(i*ydiv)).intValue());
			NumberFormat formatter = NumberFormat.getInstance();
			formatter.setMinimumFractionDigits(1);
			formatter.setMaximumFractionDigits(1);
			g2d.drawString(String.valueOf(formatter.format(i)), distfromside-22,
					new Float(height-distfromside+5-(i*ydiv)).intValue());
		}
		g2d.rotate(-Math.PI/2, distfromside-25, (height/2)+15);
		g2d.drawString("GZ (m)", distfromside-25, (height/2)+15);
		g2d.rotate(Math.PI/2, distfromside-25, (height/2)+15);
		
		//Tangent
		g2d.setColor(Color.WHITE);
		g2d.setStroke(new BasicStroke(1.2f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		g2d.drawLine(distfromside, new Float(height-distfromside+(TCG*ydiv)).intValue(), 
				distfromside+new Float((180/Math.PI)*xdiv).intValue(), 
				height-distfromside-new Float((KM-VCG-FSC)*ydiv).intValue());
		g2d.drawLine(distfromside+new Float((180/Math.PI)*xdiv).intValue(), 
				height-distfromside-new Float((KM-VCG-FSC)*ydiv).intValue(), 
				distfromside+new Float((180/Math.PI)*xdiv).intValue(), height-distfromside);
		
		//Graph
		g2d.setColor(Color.BLUE);
		g2d.setStroke(new BasicStroke(3f, BasicStroke.CAP_ROUND, BasicStroke.JOIN_ROUND));
		GeneralPath path = new GeneralPath();
		path.moveTo(distfromside, height-distfromside+(TCG*ydiv));
		for(int i = 1; i <= maxangle;i++){
			float gz = (float) (getKn(i) - (VCG * Math.sin(Math.toRadians(i))) 
			- (TCG * Math.cos(Math.toRadians(i))));
			path.lineTo((i*xdiv)+distfromside, height - distfromside- (gz*ydiv));
		}
		g2d.draw(path);
	}
	
	// Get the KN for a certain angle.
	private float getKn(float angle){
		float returnvalue = 0f;
		for (int i = 0; i < kn.length; ++i) {
			if (angle < kn[i][0]) {
				returnvalue = interpolate(angle,kn[i-1][0],kn[i][0],kn[i-1][1],kn[i][1]);
				break;
			} else if (angle == kn[i][0]) {
				returnvalue = kn[i][1];
				break;
			}
		}
		return returnvalue;
	}
	

	// Method used to interpolate between value
	// refnum is the number for which we want the value
	// refnum is between refsmall and reflarge which correspond to solsmall and
	// solllargo
	private float interpolate(float refnum, float refsmall,
			float reflarge, float solsmall, float sollarge) {
		float x = refnum - refsmall;
		float y = reflarge - refsmall;
		float z = sollarge - solsmall;
		float solution = (x / y) * z;
		return (solution + solsmall);
	}
}
