//: be/optiloading/tank/TankPanel.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.tank;

import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.ResourceBundle;
import java.util.prefs.Preferences;

import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingConstants;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import javax.swing.border.TitledBorder;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import be.optiloading.cargo.Cargo;
import be.optiloading.helpers.table.NumberEditor;
import be.optiloading.helpers.table.NumberRenderer;
import be.optiloading.helpers.table.PercentageProgressRenderer;
import be.optiloading.helpers.table.PercentageRenderer;
import be.optiloading.helpers.table.PercentageSpinnerEditor;
import be.optiloading.settings.Settings;
import be.optiloading.ship.ShipData;

/**
 * TankPanel which displays visually displays the TankList in a Table.
 * 
 * @author Tim Pintens, info@optiloading.be
 * @author <a target="_blank"
 *         href="http://www.optiloading.be">www.optiloading.be</a>
 */

@SuppressWarnings("serial")
public class TankPanel extends JPanel {
	private JTable table;
	private JScrollPane scroll;
	private Preferences prefs;
	private boolean progressbar;
	private String[] columnToolTips;
	private boolean cargo;
	private boolean ballast;
	private boolean editable;

	/**
	 * Constructor to create the GUI components and set the TableModel
	 */
	public TankPanel(String description, boolean editable, boolean cargo, 
			boolean ballast, TankList tanklist){
		super(new GridLayout(1, 0));
		this.cargo = cargo;
		this.ballast = ballast;
		this.editable = editable;
		prefs = Preferences.userNodeForPackage(getClass());
		if(cargo){
			progressbar = prefs.getBoolean("progressbarcargo", true);
		} else {
			progressbar = prefs.getBoolean("progressbarballast", true);
		}
		setBorder(new CompoundBorder(new TitledBorder(new EtchedBorder(),
				description), new EmptyBorder(5, 5, 0, 5)));

		ResourceBundle languageResource = Settings.getInstance().getResourceBundle();
		String[] normaltanks = {"compartmentTooltip","volumeTooltip2",
				"densityTooltip2","percentageFullTooltip","weightTooltip2",
				"ullageTooltip","maxPercentageTooltip","maxVolumeTooltip",
				"vcgTooltip","lcgTooltip","tcgTooltip","fsmTooltip"};
		String[] cargotanks = {"compartmentTooltip","cargoidTooltip2","volumeTooltip2",
				"densityTooltip2","percentageFullTooltip","weightTooltip2","ullageTooltip"
				,"maxPercentageTooltip","maxVolumeTooltip","vcgTooltip","lcgTooltip"
				,"tcgTooltip","fsmTooltip"};
		String[] uselist = null;
		if(cargo){
			uselist = cargotanks;
			columnToolTips = new String[cargotanks.length];
			
		} else {
			uselist = normaltanks;
			columnToolTips = new String[normaltanks.length];
		}
		for(int i=0; i < uselist.length;i++){
			columnToolTips[i] = languageResource.getString(uselist[i]);
		}
		
		if((cargo) && (!editable)){
			columnToolTips[4] = languageResource.
				getString("percentageFullTooltipClick");
		} else if((!cargo) && (!editable)){
			columnToolTips[3] = languageResource.
				getString("percentageFullTooltipClick");
		}
			
		table = new JTable(new TankModel(tanklist,editable,cargo)){
		    public Component prepareRenderer(TableCellRenderer renderer, int row, int column){
		        Component returnComp = super.prepareRenderer(renderer, row, column);
		        Color alternateColor = new Color(236,243,254);
		        Color whiteColor = Color.WHITE;
		        if (!returnComp.getBackground().equals(getSelectionBackground())){
		            Color bg = (row % 2 == 0 ? alternateColor : whiteColor);
		            returnComp .setBackground(bg);
		            bg = null;
		        }
		       
		        int progresscolumn = -1;
		        if(((TankModel) table.getModel()).isCargo() && 
		        		progressbar){
		        	progresscolumn = 4;
		        } else if(progressbar){
		        	progresscolumn = 3;
		        }
		        
		        if((table.getSelectedRow() == row) && (column != progresscolumn) && 
		        		(!((TankModel) table.getModel()).isCellEditable(row, column))){
		        	returnComp.setForeground(Color.white);
		        }
		        
		        if((table.getSelectedRow() == row) && 
		        		(((TankModel) table.getModel()).isCellEditable(row, column))){
		        	returnComp.setForeground(Color.white);
		        }
		        
		        if((table.getSelectedRow() != row) && (column != progresscolumn) &&
		        	(!((TankModel) table.getModel()).isCellEditable(row, column))){
		        	returnComp.setForeground(Color.black);
		        }
		        
		        if((table.getSelectedRow() != row) &&
			        	(((TankModel) table.getModel()).isCellEditable(row, column))){
			        	returnComp.setForeground(new Color(9,42,212));
			    }
		        
		        if((((TankModel) table.getModel()).isCellEditable(row, column)) && 
		        		(table.getSelectedColumn() != column) && 
		        		(table.getSelectedRow() != row)){
		        	returnComp.setForeground(new Color(9,42,212));
		        }

		        if (returnComp instanceof JComponent) {
		        	progresscolumn = 3;
			        if(((TankModel) table.getModel()).isCargo()){
			        	progresscolumn = 4;
			        }
		        	if((((TankModel) table.getModel()).isCellEditable(row, column))){
		        		JComponent jc = (JComponent) returnComp;
		                jc.setToolTipText(Settings.getInstance().
		                		getResourceBundle().getString("clickEdit"));
			        } else if((column == 1) && 
			        		(((TankModel) table.getModel()).isCargo())){
			        	JComponent jc = (JComponent) returnComp;
			        	DefaultTableCellRenderer render = (DefaultTableCellRenderer) jc;
			        	String id = render.getText();
			        	if(!id.equals(null)){
			        		String tip = null;
				        	for(Cargo x: ShipData.getInstance().getCargoList()){
				        		if(x.getCargoid().equals(id)){
				        			tip = x.getDescription();
				        			break;
				        		}
				        	}
				        	if(tip != null){
				        		jc.setToolTipText(tip);
				        	}
			        	} else {
			        		jc.setToolTipText(null);
			        	}
			        } else if(column == progresscolumn){
			        	JComponent jc = (JComponent) returnComp;
		                jc.setToolTipText(Settings.getInstance().
		                		getResourceBundle().getString("switchRendererToolTip"));
			        }
	            }
		        
		        return returnComp;
			}
		    
		    public Component prepareEditor(TableCellEditor editor,int row, int column){
		    	Component returnComp = super.prepareEditor(editor, row, column);
		    	returnComp.setForeground(new Color(9,42,212));
		    	return returnComp;
		    }
		    
		    protected JTableHeader createDefaultTableHeader() {
		        return new JTableHeader(columnModel) {
		            public String getToolTipText(MouseEvent e) {
		                java.awt.Point p = e.getPoint();
		                int index = columnModel.getColumnIndexAtX(p.x);
		                int realIndex = 
		                        columnModel.getColumn(index).getModelIndex();
		                return columnToolTips[realIndex];
		            }
		        };
		    }
		};
		
		table.addFocusListener(new FocusAdapter(){
			public void focusLost(FocusEvent e) {
				table.getSelectionModel().clearSelection();
			}
		});
		
		table.addMouseListener(new MouseAdapter(){
			public void mousePressed(MouseEvent me) {
				 Point p = me.getPoint();
				 int column = table.columnAtPoint(p);
				 int row = table.rowAtPoint(p);
				 int progresscolumn = 3;
			     if(((TankModel) table.getModel()).isCargo()){
			      	progresscolumn = 4;
			     }
			     if((column == progresscolumn) &&
			    		 (!((TankModel) table.getModel()).isCellEditable(row, column))){
			    	 switchProgressRenderer();
			     }
			     table.changeSelection(row, column, false, false);
			}
		});
		
		table.getTableHeader().setReorderingAllowed(false);
		((DefaultTableCellRenderer)table.getTableHeader().getDefaultRenderer())
			.setHorizontalAlignment(SwingConstants.CENTER);
		
		scroll = new JScrollPane(table,
				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		scroll.addComponentListener(new ScrollResizeListener());
		
		table.setDefaultRenderer(Float.class, new NumberRenderer(2));
		
		TableColumn col = null;
		
		int j = 1;
		if(cargo){
			j = 2;
			col = table.getColumnModel().getColumn(0);
			col.setCellRenderer(new DefaultTableCellRenderer());
		}
		if (editable){
			table.setDefaultEditor(Float.class, new NumberEditor(2));
			col = table.getColumnModel().getColumn(j+2);
			col.setCellEditor(new PercentageSpinnerEditor());
			col.setCellRenderer(new PercentageRenderer());
			col= table.getColumnModel().getColumn(j+1);
			col.setCellEditor(new NumberEditor(3));
			col.setCellRenderer(new NumberRenderer(3));
			for(int i1 = 6; i1 < 11;i1++){
				col= table.getColumnModel().getColumn(i1+j);
				col.setCellRenderer(new NumberRenderer(2));
			}
		} else {
			col = table.getColumnModel().getColumn(j+2);
			if(progressbar){
				col.setCellRenderer(new PercentageProgressRenderer());
			} else {
				col.setCellRenderer(new PercentageRenderer());
			}
			col= table.getColumnModel().getColumn(j+1);
			col.setCellRenderer(new NumberRenderer(3));
		}
		col = table.getColumnModel().getColumn(j+5);
		col.setCellRenderer(new PercentageRenderer());
		col.setCellEditor(new PercentageSpinnerEditor());
		
		if(!editable){
			table.getTableHeader().addMouseListener(new MouseAdapter(){

				public void mousePressed(MouseEvent e) {
					int j = 1;
					if(getCargo()){
						j = 2;
					}
					java.awt.Point p = e.getPoint();
	                int index = table.getColumnModel().getColumnIndexAtX(p.x);
	                int realIndex = 
	                	table.getColumnModel().getColumn(index).getModelIndex();
	                if(realIndex == (j+2)){
	                	switchProgressRenderer();
	                }
					
				}
			});
		}
		
		setPreferredWidths();
		
		JTableHeader header = table.getTableHeader();
		Dimension dim = header.getPreferredSize();
		dim.height +=5;
		header.setPreferredSize(dim);

		table.setRowHeight(22);
		add(scroll);
	}
	
	/**
	 * Update tank panel.
	 */
	public void update(){
		((TankModel) table.getModel()).fireTableDataChanged();
	}
	
	/**
	 * Lock tank panel.
	 */
	public void lock(){
		((TankModel) table.getModel()).setEditable(false);
	}
	
	/**
	 * Unlock tank panel.
	 */
	public void unlock(){
		((TankModel) table.getModel()).setEditable(true);
	}
	
	public void switchProgressRenderer(){
		int j = 1;
		if(cargo){
			j = 2;
		}
		if(progressbar){
			TableColumn col = table.getColumnModel().getColumn(j+2);
			col.setCellRenderer(new PercentageRenderer());
			progressbar = false;	
		} else {
			TableColumn col = table.getColumnModel().getColumn(j+2);
			col.setCellRenderer(new PercentageProgressRenderer());
			progressbar = true;
		}
		if(cargo){
			prefs.putBoolean("progressbarcargo", progressbar);
		} else {
			prefs.putBoolean("progressbarballast", progressbar);
		}
		update();
	}
	
	private boolean getCargo(){
		return cargo;
	}
	
	// Resize tanklist
    private class ScrollResizeListener extends ComponentAdapter {
		public void componentResized(ComponentEvent e) {
			if (table.getMinimumSize().width <= scroll.getViewport()
					.getExtentSize().width) {
				setPreferredWidths();
				table.setAutoResizeMode(JTable.AUTO_RESIZE_ALL_COLUMNS);
			} else {
				for(int i = 0; i < table.getColumnCount(); i++ ){
					TableColumn col = table.getColumnModel().getColumn(i);
					col.setPreferredWidth(col.getMinWidth());
				}
				table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
			}
		}
	}
    
    // Set prefered column widths
    private void setPreferredWidths(){
    	int j = 1;
		if(cargo){
			j = 2;
		}
    	// Compartment
		TableColumn col = table.getColumnModel().getColumn(0);
		if(ballast || cargo){
			col.setMinWidth(100);
			col.setPreferredWidth(105);
		} else {
			col.setMinWidth(110);
			col.setPreferredWidth(130);
		}
		// Cargo id
		col = table.getColumnModel().getColumn(1);
		col.setMinWidth(33);
		col.setPreferredWidth(55);
		// Vol
		col = table.getColumnModel().getColumn(j);
		col.setMinWidth(55);
		col.setPreferredWidth(75);
		// Dens
		col = table.getColumnModel().getColumn(j+1);
		col.setMinWidth(50);
		col.setPreferredWidth(95);
		//% full
		col = table.getColumnModel().getColumn(j+2);
		col.setMinWidth(60);
		if(!editable){
			col.setPreferredWidth(90);
		} else {
			col.setPreferredWidth(60);
		}
		// Weight
		col = table.getColumnModel().getColumn(j+3);
		col.setMinWidth(55);
		col.setPreferredWidth(60);
		// Ullage
		col = table.getColumnModel().getColumn(j+4);
		col.setMinWidth(45);
		col.setPreferredWidth(60);
		// Max %
		col = table.getColumnModel().getColumn(j+5);
		col.setMinWidth(60);
		col.setPreferredWidth(60);
		// Max vol.
		col = table.getColumnModel().getColumn(j+6);
		col.setMinWidth(60);
		col.setPreferredWidth(83);
		// VCG
		col = table.getColumnModel().getColumn(j+7);
		col.setMinWidth(50);
		col.setPreferredWidth(50);
		// LCG
		col = table.getColumnModel().getColumn(j+8);
		col.setMinWidth(50);
		col.setPreferredWidth(50);
		// TCG
		col = table.getColumnModel().getColumn(j+9);
		col.setMinWidth(50);
		col.setPreferredWidth(50);
		// FSM
		col = table.getColumnModel().getColumn(j+10);
		col.setMinWidth(50);
		col.setPreferredWidth(55);
    }
}