

package be.optiloading.test;




public class TestAnnealing {
	//private float[] tankVolumes = {400,500,200,300,600,100};
	//private float[] cargoVolumes = {800,500,700};
	private float[] tankVolumes = {400,200,1000,700,400,800,300,500,200,300,600};
	private float[] cargoVolumes = {2000,1500,900};
	private float totalCargoVolumes;
	private float totalTankVolumes;
	private float meanCapacity;
	private int i;
	private int j;
	
	//Annealing settings
	private float tempReductionFactor = 0.9f;
	private int maxAnnealSteps = 100;
	private int swapsPerStep = 20;
	private int trailsPerStep = 300;
	private float tempFactor = 400;
	private float temperature;
	private Solution sol;
	private Solution bestSol;
	private float bestSolEnergy;
	private float solEnergy;
	
	public static void main(String[] args) {
		new TestAnnealing().run();
	}
	
	public void run(){
		i = cargoVolumes.length;
		j = tankVolumes.length;
		totalCargoVolumes = 0f;
		totalTankVolumes = 0f;
		for(int k = 0; k < j; k++){
			totalTankVolumes += tankVolumes[k];
		}
		for(int k = 0; k < i; k++){
			totalCargoVolumes += cargoVolumes[k];
		}
		meanCapacity = totalTankVolumes/j;
		/**
		sol = new Solution();
		while(true){
			System.out.println(sol);
			pressRet();
			sol.movetoNeighbor();
		}
		*/
		//Begin simulated annealing
		long begintime = (long) System.currentTimeMillis();
		sol = new Solution();
		bestSol = (Solution) sol.clone();
		bestSolEnergy = sol.getEnergy();
		temperature = (totalCargoVolumes/totalTankVolumes)*tempFactor;
		for(int k= 0; k < maxAnnealSteps;k++){
			if(sol.anneal(temperature, trailsPerStep, swapsPerStep)){
				temperature *= tempReductionFactor;
			} else {
				break;
			}
			
			solEnergy = sol.getEnergy();
			
			if(solEnergy < bestSolEnergy){
				bestSol = (Solution) sol.clone();
				bestSolEnergy = sol.getEnergy();
			}
		}
		System.out.println(bestSol);
		System.out.println("Elapsed: "+(System.currentTimeMillis()-begintime)/1000f);
		
	}
	/**
	private static void pressRet() {
		System.out.print("\n[Press return to continue or type 'q' to quit] ");
		BufferedReader br = new BufferedReader(new InputStreamReader(System.in));
		try {
			String line = br.readLine();
			if (line == null || "q".equals(line.trim())) {
				System.out.println("Test terminated.");
				System.exit(0);
			}
		} 
		catch (IOException e) {
			System.exit(0);
		}
		System.out.println("");
	}
	*/
	class Solution implements Cloneable{
		float[] distribution;
		int[] cargoid;
		float[] slackCargo;
		
		public Solution(){
			distribution = new float[j];
			cargoid = new int[j];
			for(int k = 0; k < j; k++){
				distribution[k] = 0;
				cargoid[k] = -1;
			}
			slackCargo = cargoVolumes.clone();
			generateInitial();
		}
		
		public boolean anneal(float temp, int nlimit, int glimit){	
			float[] currentDist, currentSlackCargo;
			int[] currentCargoid;
			float currentEnergy, trailEnergy, delta, p, m;
			boolean improvement = false;
			int swaps = 0;
			
			for(int k = 0; k < nlimit; k++){
				currentDist = distribution.clone();
				currentCargoid = cargoid.clone();
				currentSlackCargo = slackCargo.clone();
				currentEnergy = getEnergy();
				
				movetoNeighbor();
				trailEnergy = getEnergy();
				delta = currentEnergy - trailEnergy;
				
				if(delta > 0f){
					improvement = true;
					swaps++;
				} else {
					p = (float) Math.random();
					m = (float) Math.exp(delta/temp);
					
					if(p < m){
						improvement = true;
						swaps++;
					} else {
						distribution = currentDist;
						cargoid = currentCargoid;
						slackCargo = currentSlackCargo;
					}
				}
				
				if(swaps > glimit){
					break;
				}
			}
			
			return improvement;
		}
		
		private void generateInitial(){
			while(!checkEveryTankLoaded() && !checkSlackCargo()){
				int maxSlackCargo = getMax(slackCargo);
				int randomTank = (int) (Math.random()*j);
				if(cargoid[randomTank] == -1){
					if(tankVolumes[randomTank] <= slackCargo[maxSlackCargo]){
						distribution[randomTank] = tankVolumes[randomTank];
						cargoid[randomTank] = maxSlackCargo;
						slackCargo[maxSlackCargo] -= tankVolumes[randomTank];
					} else {
						distribution[randomTank] = slackCargo[maxSlackCargo];
						cargoid[randomTank] = maxSlackCargo;
						slackCargo[maxSlackCargo] = 0;
					}
				}
			}
		}
		
		public float getEnergy(){
			float returnvalue = 0f;
			for(int k = 0; k < j; k++){
				if((distribution[k] != 0f) || (distribution[k] == tankVolumes[k])){
					returnvalue += (tankVolumes[k] - distribution[k]);
					returnvalue += meanCapacity;
				}
			}
			for(int k = 0; k< i; k++){
				if(slackCargo[k] != 0){
					returnvalue += (cargoVolumes[k] - slackCargo[k]);
				}
			}
			returnvalue -= (totalTankVolumes-totalCargoVolumes);
			return returnvalue;
		}
		
		public void movetoNeighbor(){
			int rand1 = (int) (Math.random()*j);
			int rand2 = (int) (Math.random()*j);
			while(rand1 == rand2){
				rand1 = (int) (Math.random()*j);
				rand2 = (int) (Math.random()*j);
			}
			/**
			System.out.println(rand1+" <-> "+rand2);
			*/
			switchTanks(rand1,rand2);
			
			boolean remslack = true;
			boolean distslack = true;
			while(remslack || distslack){
				remslack = removeSlackTanks();
				distslack = distributeSlack();
			}
		}
		
		private void switchTanks(int j1, int j2){
			float tempdist = distribution[j1];
			int tempcargoid = cargoid[j1];
			distribution[j1] = distribution[j2];
			cargoid[j1] = cargoid[j2];
			distribution[j2] = tempdist;
			cargoid[j2] = tempcargoid;
			if(distribution[j1] > tankVolumes[j1]){
				slackCargo[cargoid[j1]] += (distribution[j1] - tankVolumes[j1]);
				distribution[j1] = tankVolumes[j1];
			}
			if(distribution[j2] > tankVolumes[j2]){
				slackCargo[cargoid[j2]] += (distribution[j2] - tankVolumes[j2]);
				distribution[j2] = tankVolumes[j2];
			}
		}
		
		private boolean distributeSlack(){
			boolean returnvalue = false;
			for(int k = 0; k < i;k++){
			if(slackCargo[k] !=0){
				for(int k2 = 0; k2 < j;k2++){
					if((cargoid[k2] == k) && 
							(distribution[k2] != tankVolumes[k2])){
						/**
						System.out.println(this);
						System.out.println("distSlack " +k +" ->"+k2);
						*/
						if((tankVolumes[k2] - distribution[k2])<=slackCargo[k]){
							slackCargo[k] -= (tankVolumes[k2] - distribution[k2]);
							distribution[k2] = tankVolumes[k2];
							returnvalue = true;
						} else {
							distribution[k2] += slackCargo[k];
							slackCargo[k] = 0;
							returnvalue = true;
							break;
						}
					}
				}
			}
			}
			while(!checkEveryTankLoaded() && !checkSlackCargo()){
				int maxSlackCargo = getMax(slackCargo);
				int appropriateTank = getAppropriateEmptyTank(slackCargo[maxSlackCargo]);
				/**
				System.out.println(this);
				System.out.println("distSlack " +maxSlackCargo +" ->"+appropriateTank);
				*/
				if(slackCargo[maxSlackCargo] > tankVolumes[appropriateTank]){
					distribution[appropriateTank] = tankVolumes[appropriateTank];
					slackCargo[maxSlackCargo] -= tankVolumes[appropriateTank];
					cargoid[appropriateTank] = maxSlackCargo;
					returnvalue = true;
				} else {
					distribution[appropriateTank] += slackCargo[maxSlackCargo];
					slackCargo[maxSlackCargo] = 0;
					cargoid[appropriateTank] = maxSlackCargo;
					returnvalue = true;
				}
			}
			return returnvalue;
		}
		
		private int getAppropriateEmptyTank(float vol){
			float currentMin = 1000000000;
			int returnvalue = -1;
			for(int k = 0; k < j; k++){
				if((cargoid[k] == -1) && (Math.abs(tankVolumes[k]-vol)<currentMin)){
					returnvalue = k;
					currentMin = tankVolumes[k]-vol;
				}
			}
			return returnvalue;
		}
		
		private boolean removeSlackTanks(){
			boolean returnvalue = false;
			for(int k =0; k < j;k++){
				if((distribution[k] != tankVolumes[k]) && (distribution[k] != 0f)){
					int[] sameid = getCargoIdTanks(cargoid[k]);
					if(sameid.length != 1){
						int randomsec = getAppropriateCombTank(k);
						/**
						System.out.println(this);
						System.out.println("Comb tanks "+k + "<->" + randomsec);
						*/
						combineTanks(k,randomsec);
						returnvalue = true;
					}
					
					
				}
			}
			return returnvalue;
		}
		
		private int getAppropriateCombTank(int j1){
			int returnvalue = 0;
			float bestcomb = 1000000000;
			float comb = 0;
			for(int k = 0; k< j;k++){
				if((k != j1) && (cargoid[k] == cargoid[j1])){
					if((tankVolumes[j1]-distribution[j1])>=distribution[k]){
						comb = (tankVolumes[k]/meanCapacity);
					} else if((tankVolumes[k]-distribution[k])>=distribution[j1]){
						comb = (tankVolumes[j1]/meanCapacity);
					} else if ((tankVolumes[k] == distribution[k]) && 
							((tankVolumes[j1]-distribution[j1])<tankVolumes[k])){
						comb = 100000000;
					} else if ((tankVolumes[j1] == distribution[j1]) && 
							((tankVolumes[k]-distribution[k])<tankVolumes[j1])){
						comb = 100000000;
					} else {
						comb = tankVolumes[j1] + tankVolumes[k] 
						         - distribution[k] - distribution[j1];
					}
					if(comb < bestcomb){
						bestcomb = comb;
						returnvalue = k;
					}
					
				}
			}
			return returnvalue;
		}
		
		private void combineTanks(int j1, int j2){
			if(((tankVolumes[j1]-distribution[j1])>=distribution[j2]) &&
					((tankVolumes[j2]-distribution[j2])>=distribution[j1])){
				if((tankVolumes[j1]-distribution[j1]-distribution[j2])>
					(tankVolumes[j2]-distribution[j2]-distribution[j1])){
					distribution[j2] += distribution[j1];
					distribution[j1] = 0;
					cargoid[j1] = -1;
					return;
				} else {
					distribution[j1] += distribution[j2];
					distribution[j2] = 0;
					cargoid[j2] = -1;
					return;
				}
			}
			if((tankVolumes[j1]-distribution[j1])>=distribution[j2]){
				distribution[j1] += distribution[j2];
				distribution[j2] = 0;
				cargoid[j2] = -1;
				return;
			} else if((tankVolumes[j2]-distribution[j2])>=distribution[j1]){
				distribution[j2] += distribution[j1];
				distribution[j1] = 0;
				cargoid[j1] = -1;
				return;
			} else if(tankVolumes[j1] <= tankVolumes[j2]){
				distribution[j1] -= (tankVolumes[j2]-distribution[j2]);
				distribution[j2] = tankVolumes[j2];
				return;
			} else {
				distribution[j2] -= (tankVolumes[j1]-distribution[j1]);
				distribution[j1] = tankVolumes[j1];
				return;
			}
		}
		
		private int[] getCargoIdTanks(int id){
			int count = 0;
			for(int k =0; k < j;k++){
				if((cargoid[k] == id) && (distribution[k] != tankVolumes[k])){
					count++;
				}
			}
			int[] returnvalue = new int[count];
			count = 0;
			for(int k = 0; k < j; k++){
				if((cargoid[k] == id) && (distribution[k] != tankVolumes[k])){
					returnvalue[count] = k;
					count++;
				}
			}
			return returnvalue;
		}
		
		public String toString(){
			String returnvalue = "";
			for(int k = 0; k < j; k++){
				returnvalue += (String.valueOf(distribution[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < j; k++){
				returnvalue += (String.valueOf(tankVolumes[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < j; k++){
				returnvalue += (String.valueOf(cargoid[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < i; k++){
				returnvalue += (String.valueOf(slackCargo[k]) + "\t");
			}
			returnvalue += "\n";
			for(int k = 0; k < i; k++){
				returnvalue += (String.valueOf(cargoVolumes[k]) + "\t");
			}
			returnvalue += "\n";
			returnvalue += (String.valueOf(getEnergy()));
			
			return returnvalue;
		}
		
		private int getMax(float[]array){
			float currentmax = 0f;
			int returnvalue = -1;
			for(int k = 0; k < array.length; k++){
				if(array[k]>= currentmax){
					returnvalue = k;
					currentmax = array[k];
				}
			}
			return returnvalue;
		}
		
		private boolean checkSlackCargo(){
			float total = 0f;
			for(int k = 0; k< slackCargo.length;k++){
				total += slackCargo[k];
			}
			if(total == 0f){
				return true;
			} else {
				return false;
			}
		}
		
		private boolean checkEveryTankLoaded(){
			boolean returnvalue = true;
			for(int k = 0; k< distribution.length; k++){
				if(cargoid[k] == -1){
					returnvalue = false;
				}
			}
			return returnvalue;
		}
		
		public Object clone(){
			try {
				Object clone = super.clone();
				((Solution) clone).distribution = distribution.clone();
				((Solution) clone).slackCargo = slackCargo.clone();
				((Solution) clone).cargoid = cargoid.clone();
				return clone;
			} catch (CloneNotSupportedException e){
				throw new InternalError();
			}
		}
	}
}

