package be.optiloading.test;
import javax.swing.JOptionPane;

import org.gnu.glpk.*;


/** Beslissingsvariabelen
var x1 >=0;  // product A
var x2 >=0;  // product B

Doelfunctie
maximize z: 3*x1 + 2*x2;

Randvoorwaarden
subject to Laadtijd (p): 0.2*x1 + 0.1*x2 <= 10;
subject to Laadcapaciteit (q): x1 + x2 <= 80;
subject to Vraag (r): x1 <= 40;

end; */

public class TestGlpk {
	private void run(){
		GlpkSolver solver = new GlpkSolver();
		Hook hook = new Hook();
		solver.enablePrints(true);
		solver.setHook(hook);
		
	    solver.setProbName("Product A or B");
	    solver.setObjDir(GlpkSolver.LPX_MAX);
	    solver.addRows(3);
	    solver.setRowName(1, "p");
	    solver.setRowBnds(1, GlpkSolver.LPX_UP, 0, 10);
	    solver.setRowName(2, "q");
	    solver.setRowBnds(2, GlpkSolver.LPX_UP, 0, 80);
	    solver.setRowName(3, "r");
	    solver.setRowBnds(3, GlpkSolver.LPX_UP, 0, 40);
	    solver.addCols(2); 
	    solver.setColName(1, "x1"); 
	    solver.setColBnds(1, GlpkSolver.LPX_LO, 0, 0); 
	    solver.setObjCoef(1, 3);
	    solver.setColName(2, "x2");
	    solver.setColBnds(2, GlpkSolver.LPX_LO, 0, 0); 
	    solver.setObjCoef(2, 2);
	    int[] ia = new int[7];
	    int[] ja = new int[7];
	    double[] ar = new double[7];
	    
	    ia[5] = 1;
	    ja[5] = 1; 
	    ar[5] = 0.2;
	    
	    ia[4] = 1;
	    ja[4] = 2; 
	    ar[4] = 0.1;
	    
	    ia[3] = 2;
	    ja[3] = 1; 
	    ar[3] = 1;
	    
	    ia[2] = 2;
	    ja[2] = 2; 
	    ar[2] = 1;
	    
	    ia[1] = 3;
	    ja[1] = 1; 
	    ar[1] = 1;
	    
	    solver.loadMatrix(5, ia, ja, ar);
	    solver.simplex();
	    double Z = solver.getObjVal();
	    double x1 = solver.getColPrim(1);
	    double x2 = solver.getColPrim(2);
	    System.out.println("Z = "+Z +" x1 = "+ x1 +" x2 = "+ x2);
	    solver.deleteProb();
	}
	
	public static void main(String[] args){
		new TestGlpk().run();
	}
	
	public class Hook implements GlpkHookIFC{

		public void fault(String s) {
			JOptionPane.showMessageDialog(null,s);	
		}

		public void print(String s) {
			JOptionPane.showMessageDialog(null,s);
		}
		
	}
}
