//: /be/optiloading/test/TestXML.java
/*
 * This work is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This work is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * Copyright (c) 2007 Tim Pintens. All rights reserved.
 */

package be.optiloading.test;

import java.io.File;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

public class TestXML {
	private Document document;
	private XPath xPath;

	public static void main(String[] args) {
		if (args[0] != null) {
			TestXML data = new TestXML(args[0]);
			System.out.println(data.get("km", "draught", "12.054564"));
		} else {
			System.out.println("Usage java TestXML filename");
		}

	}

	// Build a DOM Document object and initialize the XPath object
	TestXML(String file) {
		try {
			DocumentBuilderFactory docfactory = DocumentBuilderFactory
					.newInstance();
			File xmlFile = new File(file);
			DocumentBuilder builder = docfactory.newDocumentBuilder();
			document = builder.parse(xmlFile);

			XPathFactory factory = XPathFactory.newInstance();
			xPath = factory.newXPath();

		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	/**
	 * Method used to get data.
	 * 
	 * @param what
	 *            String which defines which kind of data we are interested in
	 * @param reference
	 *            String which defines which kind of refenrence data we will
	 *            use.
	 * @param refvalue
	 *            String that indicates the value of the reference data.
	 */
	public double get(String what, String reference, String refvalue) {
		double returnvalue = 0;

		try {
			// Using XPath get the nodes where the refvalue is in between
			// The expression forms a union between the last value of all values
			// lower than the refvalue and
			// the first value of all values higher than the refvalue
			NodeList nodes = (NodeList) xPath.evaluate("/hydrostatic/data["
					+ reference + " <=" + refvalue
					+ "][last()] | /hydrostatic/data[" + reference + " >= "
					+ refvalue + "][1]", document, XPathConstants.NODESET);

			// If the refvalue is exactly a value that is included in the data
			// then
			// there is no need to interpolate and there will only be one node
			// found
			if (nodes.getLength() == 1) {
				Element data = (Element) nodes.item(0);
				String ret = data.getElementsByTagName(what).item(0)
						.getFirstChild().getNodeValue();
				returnvalue = Double.valueOf(ret);
			} else {
				Element data = (Element) nodes.item(0);
				Element data2 = (Element) nodes.item(1);
				String refsmall = data.getElementsByTagName(reference).item(0)
						.getFirstChild().getNodeValue();
				String reflarge = data2.getElementsByTagName(reference).item(0)
						.getFirstChild().getNodeValue();
				String solsmall = data.getElementsByTagName(what).item(0)
						.getFirstChild().getNodeValue();
				String sollarge = data2.getElementsByTagName(what).item(0)
						.getFirstChild().getNodeValue();
				returnvalue = interpolate(Double.valueOf(refvalue), Double
						.valueOf(refsmall), Double.valueOf(reflarge), Double
						.valueOf(solsmall), Double.valueOf(sollarge));
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return returnvalue;
	}

	/**
	 * Method used to interpolate between value refnum is the number for which
	 * we want the value. refnum is between refsmall and reflarge the value
	 * (sollution) we are looking for which coresponds to the refnum value is
	 * betweeb solsmall and solllarge for instance if we had following table and
	 * were interested in what 15 would correspond to 10 -> 1 20 -> 2
	 * interpolate(15,10,20,1,2) = 1.5
	 */
	private static double interpolate(double refnum, double refsmall,
			double reflarge, double solsmall, double sollarge) {
		double x = refnum - refsmall;
		double y = reflarge - refsmall;
		double z = sollarge - solsmall;
		double solution = (x / y) * z;
		return (solution + solsmall);
	}
}
